/*************************************************************************
  bbINFO by Brent Beyeler, beyeler@home.com
*************************************************************************/

/*                                                           
bbINFO2 is an MPEG transport and program stream analyzer

Usage: bbINFO2  MPEG filename  <verbose level 0..4, default = 1>
       verbose levels 3..4 only available for program streams

Examples:
  To show summary info only
     bbINFO2 test.mpg 0

  To list the timing information for all packets
     bbINFO2 test.mpg 1

  To list all packets in detail (can generate huge files)
     bbINFO2 test.mpg 2

  To list first 99 packets in detail
     bbINFO2 test.mpg 2 99

  To list all packets types visually saving to file test.txt
     bbINFO2 test.vob 3 > test.txt

  To list all packets types visually plus PTS, DTS and P-STD info
     bbINFO2 test.vob 4

*/

#include "bits.h"
#include <string.h>
// #include <mem.h>    BORLAND
#define setmem(a, b, c) memset(a, c, b)

#define MPEG_PIC_START_CODE      0x00000100
#define MPEG_SEQ_START_CODE      0x000001B3
#define MPEG_GOP_START_CODE      0x000001B8
#define MPEG_PROGRAM_END_CODE    0x000001B9
#define PACK_START_CODE          0x000001BA
#define SYSTEM_HEADER_START_CODE 0x000001BB
#define PACKET_START_CODE_PREFIX 0x000001

#define PROGRAM_ASSOCIATION_TABLE 0x0000
#define CONDITIONAL_ACCESS_TABLE  0x0001
#define NULL_PACKET               0x1FFF

#define TRANSPORT_SYNC_BYTE      0x47
#define PROGRAM_STREAM_MAP       0xBC
#define PRIVATE_STREAM_1         0xBD
#define PADDING_STREAM           0xBE
#define PRIVATE_STREAM_2         0xBF
#define ECM_STREAM               0xF0
#define EMM_STREAM               0xF1
#define PROGRAM_STREAM_DIRECTORY 0xFF
#define DSMCC_STREAM             0xF2
#define ITUTRECH222TYPEE_STREAM  0xF8
#define VIDEO_STREAM_0           0xE0
#define VIDEO_STREAM_F           0xEF
#define AUDIO_STREAM_0           0xC0
#define AUDIO_STREAM_1F          0xDF
#define SUBSTREAM_DDP_0          0xC0
#define SUBSTREAM_DDP_F          0xCF
#define SUBSTREAM_AC3_0          0x80
#define SUBSTREAM_AC3_8          0x87
#define SUBSTREAM_DTS_0          0x88
#define SUBSTREAM_DTS_8          0x8F
#define SUBSTREAM_PCM_0          0xA0
#define SUBSTREAM_PCM_F          0xAF
#define SUBSTREAM_SUBPIC_0       0x20
#define SUBSTREAM_SUBPIC_1F      0x3F

#define PICTURE_START_CODE       0x00
#define USER_DATA_START_CODE     0xB2
#define SEQUENCE_HEADER_CODE     0xB3
#define EXTENSION_START_CODE     0xB5
#define SEQUENCE_END_CODE        0xB7
#define GROUP_START_CODE         0xB8
#define PICTURE_CODING_EXTENSION 0x04

#define FAST_FORWARD 0
#define SLOW_MOTION  1
#define FREEZE_FRAME 2
#define FAST_REVERSE 3
#define SLOW_REVERSE 4

#define MP3_ID3_START_CODE 0x49443303

// MPA HDR - Mepg Audio Header related

//double frequency [4] = {44.1, 48, 32, 0};
unsigned int slots [4] = {12, 144, 144, 12};
unsigned int SlotBytes[4] = {4, 1, 1, 4};

// unsigned uFrequency[4] = {44100, 48000, 32000, 44100}; // ,0};
// unsigned uFreq_k[4] = {44, 48, 32, 0};

unsigned MPA_SAMPLE_HZ[4][4]
    = {
    {44100, 48000, 32000, 48000}, // Original Std
    {22050, 24000, 16000, 24000}, // Mpeg 2.5
    {22050, 24000, 16000, 24000}, // LSF Extension
    {11025, 12000,  8000, 12000}  // Mpeg 2.5 + LSF
} 
;

unsigned int uMPA_kBitRate=0;
//unsigned int BitRate_tbl [4][16] =
//    {{0,32,64,96, 128,160,192,224,  256,288,320,352,  384,416,448,0},
//     {0,32,48,56,  64, 80, 96,112,  128,160,192,224,  256,320,384,0},
//     {0,32,40,48,  56, 64, 80, 96,  112,128,160,192,  224,256,320,0},
//     {0, 0, 0, 0,   0,  0,  0,  0,    0,  0,  0,  0,    0,  0,  0,0}};


unsigned MPA_KBIT_RATE[2][3][16]
    ={
/* MPEG-1 */
  /* Layer I   */
  {{0,32,64,96,  128,160,192,224,    256,288,320,352,   384,416,448,666},
  /* Layer II  */
   {0,32,48,56,   64, 80, 96,112,    128,160,192,224,   256,320,384,666},
  /* Layer III */
   {0,32,40,48,   56, 64, 80, 96,    112,128,160,192,   224,256,320,666}},
/* MPEG-2 LSF - MPEG-2 Low Sampling Frequencies Extension */
  /* Layer I   */
  {{ 0,32,48,56,  64, 80, 96,112,    128,144,160,176,   192,224,256,666},
  /* II & III  */
   { 0, 8,16,24,  32, 40, 48, 56,     64, 80, 96,112,   128,144,160,666},
   { 0, 8,16,24,  32, 40, 48, 56,     64, 80, 96,112,   128,144,160,666}}
}
;


char MPA_EXTN_ABBR[4][4]  // L=LSF=LowSamplingFreq  .5= Mpeg 2.5 unofficial extension
    = {"", "L", ".5", ".5L"}
;


char MPG_CH_MODE_ABBR[4][4]
    = {"S ", "js", "x2", "M "}
;


// -------------
  char *szFRAME_RATE[16]
    = {"18",     // Allow for illegal value of zero
      "23.976", "24",
      "25", "29.97", "30",
      "50", "59.94", "60",
      // rest are "reserved",
      // but turn up in non-std files.  Guessing values.  see also K_FrameCode
      "12-?", "16-?", "18-?", "20-?", "6-?", "2-?", "1-?"};

  unsigned int uFRAME_RATE_1001[16]
    = {18*1001, 24*1000, 24*1001,
       25*1001, 30*1000, 30*1001,
       50*1001, 60*1000, 60*1001,
      // rest are "reserved",
      // but turn up in non-std files.  Guessing values.  see also K_FrameCode
       12*1001, 16*1001, 18*1001, 20*1001, 6*1001, 2*1001, 1*1001};


//  ------------

unsigned char cMPA_Hdr[4];
unsigned int uMPA_HdrLoc=0;

unsigned int uMPA_25_LSF=0, uMPA_Mpeg25=0, uMPA_LSF=0;  // Flags -  Mpeg2.5  and LowSamplingFreqExtension
unsigned int uMPA_Channel_ix=0;

char cMPA_Sus, szMPA_Note[32];


// --------

static void transport_stream();

static int verbose_level=0, iLimit=0, iChk=0;
unsigned int uResync_Ctr=0;
unsigned int uFromKB=0, uPrevKB=0, uSkipKB=0, uSkipB=0, uFromB=0;
static int col=0;
static bool vob_flag=0;
static char oldLevel3[80], level3[80];

unsigned int  uVidTrk=0, uRemaining=0;
unsigned int  uGOP_Code_Prev=0, uGOP_Code_Start=0;
unsigned int  uDrop, uHH, uMM, uSS, uFF;
unsigned long uPTS_Major=0;
unsigned long iSeq_ctr=0, iSeq_Unaligned_ctr=0, uGOP_ctr=0, uPIC_ctr=0;
unsigned long uRel_ctr=0, uRpt_Rel=0, uRpt_ctr=0;
unsigned long uGAP_ctr=0, uGOP_PrevPTS=0, uGOP_Diff_PTS_ms=0;
unsigned long uGOPspanDVDPAL=0, uGOPspanDVDNTSC=0, uGOPspan1=0, uGOPspan0=0;
unsigned long uGOPspanKDVD=0, uGOPspanLONG=0, uGOPspanSHORT=0, uGOPspanMISC=0;
unsigned int  uStartTC[257], uStartADJ=0, uADJ_FLag=0;
         char szFlag[64];
         char tmpStr[512];
         char szTmp32[32], szTmp16[16], szBuffer[512];
unsigned int uGOP_Frames[602];

int  bytes_used=0;

int mpeg_type=0; // 0 = none, 1 = program stream, 2 = transport stream

unsigned int  uStartCode=0; 
unsigned long stream_id=0, substream_id=0;
int PES_packet_length=0, PES_header_data_length=0;
int iPayload=0;
unsigned long stream_packets[260], stream_bytes[260];
unsigned long substream_packets[260], substream_bytes[260];

unsigned int mpeg2=0, ntsc=0;
int Pack_Version_Bits=0;

  unsigned long pack_packets=0,  system_headers=0;
  unsigned int uDVDfit=0, uVCDfit=0, uOtherPacks=0, uOrphans=0;
  unsigned long uScrExtn_Ctr=0, uErrPrevK=0;
  unsigned long uPackSize=0, uPackPics=0, uPackPrevPics=0; // , iPES_Offset, iDiff ;
  double PrevPack_bit_count;

  int iFirstPktOfSubStream=0;

  unsigned long uSCR_Prev=0,  uPTS_Prev[260],  uDTS_Prev[260];
  unsigned long uSCR_Major=0, uDTS_Major=0;
  long double   dScr_value, dPts_value, dDts_value;
  int iDiff_TC=0;


  unsigned long PTS_DTS_flags, ESCR_flag, ES_rate_flag, DSM_trick_mode_flag;
  unsigned long additional_copy_info_flag, PES_CRC_flag, PES_extension_flag;
  unsigned long trick_mode_control, PES_private_data_flag, pack_header_field_flag;
  int pack_field_length=0;
  unsigned long program_packet_sequence_counter_flag, PSTD_buffer_flag;
  unsigned long PES_extension_flag_2;
  int PES_extension_field_length=0;

  int iRemain=0;

  int offset=0, offset_Prev=0;
  int iFramesInPkt=0, iNomFrameSz=0, iFrameBytes=0, iPrevNomFrameSz=0;


  unsigned long video_sync[17];
  int audio_sync[33];

  unsigned int uHoriz, uVert, uAspectCode;
  unsigned int uFRateCode, uFPeriod, uFRate1001, uFRate_Xtn_n, uFRate_Xtn_d;
  unsigned int uPicStruct, uRepeat;
  double fFrameRate;

  unsigned int uMPA_Layer_ix=0, uMPA_Layer=0;
  unsigned int BitRate_ix=0, Freq_ix=0, padding_bit=0;

//  unsigned long ac3_bytes[8], dts_bytes[8], pcm_bytes[16], subpic_bytes[32];

int iArgPtr=0;
int iSaveWanted=0;



//--------------------------------

void zprintf(int, char *, int);
void SYSHDR_Anl();
void Field_level_PktHdr();
void PKT_TYPE_Reports();
void PACK_Anl();
void PES_HDR_Mpeg2(), PES_HDR_Mpeg1();
void GOP_Stats_Inc();
void DD_Audio_Info(char Codec_Abbr[4], int iTrackRel);
void MPA_Audio_Info();
void Hex_Prt(int);
void PS2_Info();
void PS1_Info();

void PCM_Audio_Info();
void PTS_Info(), DTS_Info();
void PTS_Mpeg1_Full(),  DTS_Mpeg1_Full();
void PTS_Mpeg1_Basic(), DTS_Mpeg1_Basic();

void PES_Minor(), PES_HDR_Flush();
void Video_Payload();

/*
void ArgQualChk()
{
  if (*argv[iArgPtr] == '/')
  {
    if (!strcmp(argv[iArgPtr], "/R")
        iSaveWanted = 1;

    iArgPtr++;
  }
}
*/


//------------------------------------------------------------

int main(int argc, char* argv[])
{
  unsigned long i;
  unsigned long uTmp1, uTmp2, uTmp6, uTmp7; // , uTmp3;
  char cTmp1;

  bool firsttime = true;


  bool streams_found[256]; //, MpegPrev;

  int iTmp1; // , iTmp2;
  int iTmp6, iTmp7, iTmp8;

//INITIALIZE WS

  for (i = 0; i < 16; i++)
  {
    video_sync[i] = 0;
    // pcm_bytes[i] = 0;
  }

  for (i = 0; i < 32; i++)
  {
    audio_sync[i] = -1;
    // subpic_bytes[i] = 0;
  }

  for (i = 0; i < 257; i++)
  {
    streams_found[i] = false;
    stream_packets[i] = 0;
    stream_bytes[i] = 0;
    substream_packets[i] = 0;
    substream_bytes[i] = 0;
  }

  //for (i = 0; i < 8; i++)
  //{
  //  ac3_bytes[i] = 0;
  //  dts_bytes[i] = 0;
  //}

  PrevPack_bit_count = 0.0;
  pack_packets = 0;
  uDVDfit = 0; uVCDfit = 0; uOrphans = 0; uOtherPacks = 0;
  uScrExtn_Ctr = 0;
  system_headers = 0;
  uGOP_ctr = iSeq_ctr = iSeq_Unaligned_ctr = uPIC_ctr = 0;
  uRel_ctr = 0; uGAP_ctr = 0;
  uRpt_Rel = 0; uRpt_ctr = 0;
  uGOP_PrevPTS = 0xFFFFFFFF;
  uGOP_Code_Prev = 0;
  col = 0;
  strcpy(level3, "");
  strcpy(oldLevel3, "");
  verbose_level = 1;
  uSCR_Prev = 0;
  setmem(&uPTS_Prev,    sizeof(uPTS_Prev), 0xFFFFFFFF);
  setmem(&uDTS_Prev,    sizeof(uDTS_Prev), 0);
  setmem(&uStartTC,     sizeof(uStartTC), 0xFFFFFFFF);
  setmem(&uGOP_Frames,  sizeof(uGOP_Frames), 0);
  uADJ_FLag = 0;

  uFRateCode = 3; uFRate1001 = 25025;  // PAL Default

  iLimit = 999999 ;
  iChk   = 0 ;

  printf("bbINFO2 - Ver 2.2c   Mods by RocketJet.     F=%d\n");
  printf("          Original by Beyeler (beyeler@home.com)\n");
  printf("          Speed increases by, Apachez and Christian Vogelgsang\n");



  //               TEST CASE
  //argc = 2;
  //strcpy(tmpStr, "_MPEG-1_.mpg");
  //verbose_level = 2;
  //iLimit = 100;
  //goto Args_X;
  


  if (argc < 2)
  {
    printf("\nbbINFO2 analyzes MPEG transport and program streams\n\n");
    printf("Usage: bbINFO2  MPEG filename  <verbose level 0..4, default = 1>\n");
    //printf("       verbose levels 3..4 only available for program streams\n");

    printf("\nExamples:\n");
    printf("  0 - SUMMARY information only\n");
    printf("      bbINFO2 test.mpg 0\n\n");

    printf("  1 - BASIC timing information for all packets\n");
    printf("      bbINFO2 test.mpg 1\n\n");

    printf("  2 - DETAIL info for first 99 packets, saving to text file\n");
    printf("      bbINFO2 test.mpg 2 99  > test.txt\n\n");

    printf("  3 - ABBREVIATED Packet Types, saving to text file\n");
    printf("      bbINFO2 test.vob 3     > test.txt\n\n");

    printf("  4 - ABBR-PLUS Packet Types including PTS, DTS and P-STD info\n");
    printf("      bbINFO2 test.vob 4\n");
   exit (1);
  }

  iArgPtr = 1;
  // ArgQualChk();

  strcpy(tmpStr, argv[iArgPtr++]);

  if (argc > iArgPtr)
  {
    sscanf(argv[iArgPtr++], "%d", &verbose_level);

    if (verbose_level < 0)
        verbose_level = 0 - verbose_level;
    //if (verbose_level < 1)
    //  verbose_level = 1;
    if (verbose_level > 4)
        verbose_level = 4;
    if (argc > iArgPtr)
    {
      sscanf(argv[iArgPtr++], "%d", &iLimit);
    }
    // ArgQualChk();

  }

//Args_X:
  printf("        - Verbose Level %d\n", verbose_level);

  init_getbits(&tmpStr[0]);

  printf("File %s\n", tmpStr);  // argv[1]);

  strlwr(tmpStr);
  vob_flag = (strstr(tmpStr, ".vob") != NULL);

// CHECK START OF FILE INFO

  mpeg_type = 0; // iPayload=0;
  while (!mpeg_type)
  {
    i = look_ahead(32);

    if ( i >= 0x01BD
    &&   i <= 0x01FF)
    {
      mpeg_type = 1;
      i = getbits(32);
      uTmp1 = (look_ahead(32) >> 14 ) & 3;

      printf("     is an MPEG-%d PES Stream\n\n",
                             uTmp1);
      if (uTmp1 == 2)
      {
        mpeg2 = true;
      }
      else
      {
        mpeg2 = false;
      }
    }
    else
    if ( i == PACK_START_CODE)
    {
      mpeg_type = 1;
      i = getbits(32);
      uTmp1 = look_ahead(2) + 1;
      printf("     is an MPEG-%d Program Stream\n\n",
                             uTmp1);
      if (uTmp1 == 2)
      {
        mpeg2 = true;
      }
      else
      {
        mpeg2 = false;
      }
    }
    else
    if (look_ahead(8) == TRANSPORT_SYNC_BYTE)
    {
        printf("       maybe MPEG-2 TRANSPORT Stream\n\n");
        if (verbose_level > 2)
        {
          printf("\n    ****\n    ****Invalid verbose level (%d) for transport streams.\n    ****\n\n", verbose_level);
          exit(1);
        }
        mpeg_type = 2;
    }
    else
    if (i == MPEG_SEQ_START_CODE
    ||  i == MPEG_GOP_START_CODE
    ||  i == MPEG_PIC_START_CODE)
    {
      printf("     is an MPEG ELEMENTARY Video Stream     StartCode=x%x\n\n** USE BBVinfo utility instead ***\n\n\n",
                                                                i);
      exit(8);
    }
    else
    if (i == MP3_ID3_START_CODE)
//    |   i == MP3_SYNC_CODE)
    {
      printf("     is an MPEG AUDIO Elementary Stream ******\n\n** USE BBAinfo utility instead ***\n\n\n");
      exit(8);
    }
    else
        getbits(8);
  }

  if (!mpeg_type)
  {
    printf("\n    ****\n    ****  UNKNOWN file type.\n    ****\n");
    exit(1);
  }

  if (mpeg_type == 2)
  {
    transport_stream();
    exit(0);
  }


  // MAIN LOOP

  do
  {
    if (!firsttime)
      i = getbits(32);
    else
      firsttime = false;

nextpass:
    uStartCode = i;
    stream_id  = i & 0x000000FF;

    switch (uStartCode)
    {
      case PACK_START_CODE:
           stream_packets[stream_id]++;
           PACK_Anl();
        break;

      case SYSTEM_HEADER_START_CODE:
           stream_packets[stream_id]++;
           SYSHDR_Anl();
        break;

      case MPEG_PROGRAM_END_CODE:
           zprintf(1, "\n\n** mpeg_program_END_code= %08X\n\n", i);
           stream_packets[stream_id]++;
           stream_bytes[stream_id] += 4;

           if (verbose_level > 2)
               strcat(level3, "E");
         break;

      default:
        if ((i >> 8) != PACKET_START_CODE_PREFIX)
        {
            uResync_Ctr++;
            stream_packets[257]++;

            uFromB =  (unsigned int)(bitcount() / 8 - 4);
            uTmp6 = uFromB/1024;
            uTmp7 = uTmp6 - uErrPrevK;
            printf("\n\n*BAD* Unknown bits:  x%06X   at RBA %d [x%06X]  Since=%uK\n",
                                          i, uFromB, uFromB, uTmp7);
            printf("\nAttempting ReSynch ... ");
            uFromKB = uFromB / 1024;
            uPrevKB = 0;
            uErrPrevK = uTmp6;
Sync_More:
            if (seek_sync(PACKET_START_CODE_PREFIX, 24))
            {
                i = 0x00000100 | getbits(8);
                if (i >= 0x0100 && i <= 0x01B8) // Picture control
                {
                  uSkipKB = (unsigned int)(bitcount() / 8192) - uFromKB;
                  if ((uSkipKB - uPrevKB) >= 1024)
                  {
                    printf("\n   Skipped %d megabytes...",
                                 (uSkipKB/1024));
                    uPrevKB = uSkipKB;
                  }
                  goto Sync_More;               // Not a pkt hdr
                }

                uTmp1 =  (unsigned int)(bitcount() / 8 - 4);
                uTmp2 = uTmp1 - uFromB;
                printf("Succesfull Resync after %d bytes\n\n",
                                                          uTmp2);
                stream_bytes[257] += uTmp2;
                goto nextpass;

            }

            if (end_bs())
            {
                uTmp1 =  (unsigned int)(bitcount() / 8 - 4);
                uTmp2 = uTmp1 - uFromB;
                stream_bytes[257] += uTmp2;
                break;
            }

          printf("\n    *BAD*\n    ***** Unknown bits in stream, could not resynch\n    *****\n\n");
          exit(1);
        }

        if (verbose_level > 1)
        {
           zprintf(1, "\npacket_start_code_prefix= %06X", i >> 8);
           iTmp1 =  (int)(bitcount() / 8 - 4);
           zprintf(1, "  RBA x%06X", iTmp1);
        }

        // stream_id = i & 0x000000FF;

        if (!streams_found[stream_id])
             streams_found[stream_id] = true;

        iFirstPktOfSubStream = (!(stream_packets[stream_id]++)) && !verbose_level;


        if (verbose_level < 3) // Field reports ?
        {
           if (verbose_level || iFirstPktOfSubStream) // Allow for Summary Report
           {
              Field_level_PktHdr();
           }
        }
        else
        {
           zprintf(1, "\n", 0);
           PKT_TYPE_Reports();
        }


        //iPES_Offset = bitcount() / 8;
        //iDiff = iPES_Offset - PrevPack_bit_count;

        PES_packet_length = getbits(16);

        zprintf(1, "  packet_length= %d", PES_packet_length);
        zprintf(1, " x%04X\n", PES_packet_length); ///* Offset=%d */, iDiff);

        switch (stream_id)
        {
          case PROGRAM_STREAM_MAP:
          case ECM_STREAM:
          case EMM_STREAM:
          case PROGRAM_STREAM_DIRECTORY:
          case DSMCC_STREAM:
          case ITUTRECH222TYPEE_STREAM:
               for (iPayload = 0; iPayload < PES_packet_length; iPayload++)
                    getbits(8);
               if (verbose_level > 2)
                   strcat(level3, "o");
                zprintf(2, "  read %d PES packet data bytes\n", iPayload);
                stream_bytes[stream_id] += iPayload;
                break;


 /*
          case PRIVATE_STREAM_2:
            for (iPayload = 0; iPayload < PES_packet_length; iPayload++)
              getbits(8);
            if (verbose_level > 2)
              strcat(level3, "2");
            zprintf(2, "  read %d PES packet data bytes\n", iPayload);
            stream_bytes[stream_id] += iPayload;
            break;
   */


          case PADDING_STREAM:
            bytes_used = 0;

            // REMOVED PTS ANALYSIS OF PADDING STREAM

            iPayload = 0;
            iRemain = PES_packet_length - bytes_used;

            if (verbose_level == 2) //   || iFirstPktOfSubStream)
                Hex_Prt(32);
                
            for (; iPayload < iRemain; iPayload++)
              getbits(8);
            if (verbose_level > 2)
              strcat(level3, "p");
            zprintf(2, "  read %d Padding_bytes\n", iPayload);
            stream_bytes[stream_id] += iPayload;
            break;

          default:
            bytes_used = 0;

            iTmp1 = look_ahead(2);

            if (!iTmp1  && mpeg2)
            {
               if (stream_id == PRIVATE_STREAM_2)
                   strcpy(szBuffer,"\n VOB NAV %d\n\n");
               else
                   strcpy(szBuffer,"\n Different Mpeg Version flags %d\n\n");

               zprintf(1,szBuffer, (iTmp1+1));
               //beep(200,200);
               fflush(stdout);
            }

            //if (iTmp1)
            //   mpeg2 = true;
            //else
            //   mpeg2 = false;

            if (mpeg2)
            {
               PES_HDR_Mpeg2();

               //if ((verbose_level == 1) && (PTS_DTS_flags != 2) && (PTS_DTS_flags != 3))
               //  printf("\n");

               if (PTS_DTS_flags == 2)
               {
                   PTS_Info();
               }
               else
               if (PTS_DTS_flags == 3)
               {
                   PTS_Info();
                   DTS_Info();
               }
               else
               if (PTS_DTS_flags == 1)
               {
                   DTS_Info();
               }

               // VOB NAV PACKETS may not conform to Mpeg2 PES HDR format
               if ((stream_id == PRIVATE_STREAM_2) ) // && (!vob_flag))
               {
                   PES_HDR_Flush();
               }
               else
                   PES_Minor();

               if (verbose_level < 3)  // RJ -  used to be ==1 but I want to see hdr desc.
               {
                 if ((stream_id >= VIDEO_STREAM_0) 
                 &&  (stream_id <= VIDEO_STREAM_F))
                 {
                    Video_Payload();
                 }

                 else
                 if ((stream_id >= AUDIO_STREAM_0) 
                 &&  (stream_id <= AUDIO_STREAM_1F))
                 {
                    MPA_Audio_Info();
                 }
                 else
                 if ((stream_id == PRIVATE_STREAM_2) ) // && (!vob_flag))
                 {
                    PS2_Info();
                 }
                 else
                 if ((stream_id == PRIVATE_STREAM_1))// && (vob_flag))
                 {
                    PS1_Info();
                 }  
                 else
                 {
                    iRemain = PES_packet_length - bytes_used; // BYTE-ME - 3;
                    iPayload = 0;
                    if (verbose_level == 2 || iFirstPktOfSubStream)
                    {
                       Hex_Prt(16);
                    }

                    for (; 
                         iPayload < iRemain; 
                         iPayload++)
                      getbits(8);
                 }

               }  // endif Verbose Level < 3

               else
               { // Verbose_Level >= 3
                 // BORING
                 if ((stream_id == PRIVATE_STREAM_1)) // && (vob_flag))
                 {
                    substream_id = getbits(8);
                    iFirstPktOfSubStream = !substream_packets[substream_id] && !verbose_level;
                 }
                 else
                     getbits(8);

                 for (iPayload = 0;
                      iPayload < PES_packet_length - PES_header_data_length - 4;
                      iPayload++)
                     {
                       getbits(8);
                     }

                 iPayload++;
               } // ENDELSE NOT Interested

            } // ENDIF Mpeg-2

            else /* mpeg-1 */
            {

              PES_HDR_Mpeg1();

              if (verbose_level < 3)
              {
                if ((stream_id >= VIDEO_STREAM_0) 
                &&  (stream_id <= VIDEO_STREAM_F))
                {
                    Video_Payload();
                }
                else
                if ((stream_id >= AUDIO_STREAM_0)
                &&  (stream_id <= AUDIO_STREAM_1F))
                {
                     MPA_Audio_Info();
                }
                else
                {
                    for (iPayload = 0; 
                         iPayload < PES_packet_length - bytes_used; 
                         iPayload++)
                         {
                            getbits(8);
                         }
                    substream_id = PRIVATE_STREAM_1; // Substreams are NOT officially defined in Mpeg-1
                }
              }
              else
              for (iPayload = 0; 
                   iPayload < PES_packet_length - bytes_used; 
                   iPayload++)
                   {
                       getbits(8);
                   }
            } // end-else NOT Mpeg-2


            if ((stream_id == PRIVATE_STREAM_1) // && (vob_flag))
            && (substream_id <= 255))
                substream_packets[substream_id]++;

            if (iPayload)
            {
              zprintf(2, "  read %d PES packet data bytes\n", iPayload);
              stream_bytes[stream_id] += iPayload;

              if ((stream_id == PRIVATE_STREAM_1)) // && (vob_flag))
              {
                substream_bytes[substream_id] += iPayload;

                //if ((substream_id >= SUBSTREAM_AC3_0) 
                // && (substream_id <= SUBSTREAM_AC3_8))
                //    ac3_bytes[substream_id - SUBSTREAM_AC3_0] += iPayload - 4;
                //if ((substream_id >= SUBSTREAM_DTS_0) 
                // && (substream_id <= SUBSTREAM_DTS_8))
                //    dts_bytes[substream_id - SUBSTREAM_DTS_0] += iPayload - 1;
                //if ((substream_id >= SUBSTREAM_PCM_0) 
                // && (substream_id <= SUBSTREAM_PCM_F))
                //    pcm_bytes[substream_id - SUBSTREAM_PCM_0] += iPayload - 1;
                //if ((substream_id >= SUBSTREAM_SUBPIC_0) 
                // && (substream_id <= SUBSTREAM_SUBPIC_1F))
                //    subpic_bytes[substream_id - SUBSTREAM_SUBPIC_0] += iPayload - 1;
              }
            }

            break;
        }
    }
    iChk++  ;

  } while (
            //(i != MPEG_PROGRAM_END_CODE) && 
            (!end_bs()) && iChk < iLimit);

  if ((verbose_level > 2) && (strlen(level3)))
    zprintf(3, level3, 0);

  if (PrevPack_bit_count == 0.0)
      uPackSize = 0;
  else
      uPackSize = (((unsigned long)(bitcount() - PrevPack_bit_count)) >> 3) + 4;
      
  //PrevPack_bit_count = bitcount();

  //if (uPackSize  != 0)
  //{
      printf("\n\nPack last size=%d ", uPackSize );
      if  (end_bs())
      {
          printf("\n\n*** END OF FILE REACHED \n");
          if (uPackSize == 2048)
              uDVDfit++;
          else
          if (uPackSize == 2324)
              uVCDfit++;
          else
          if (uPackSize == 14)
              uOrphans++;
          else
              uOtherPacks++;
      }
  //}

  printf("\n\nPkt Errors= %d\n", uResync_Ctr);

  printf("\nMPEG Packs= %u, DVDsized=%u, VCDsized=%u, Other=%u,\n     Orphans=%u, Extn=%u\n\n",
                 pack_packets, uDVDfit, uVCDfit, uOtherPacks, uOrphans, uScrExtn_Ctr);

  if (system_headers)
    printf("System headers= %4u\n", system_headers);
    
  if (iSeq_ctr)
    printf("   Seq headers= %4u  Unaligned=%d\n", iSeq_ctr, iSeq_Unaligned_ctr);

  if (uGOP_ctr < iSeq_ctr )
  {
    sprintf(szBuffer," **** DOES NOT MATCH SEQ HDR COUNT ****");
  }
  else
   szBuffer[0] = 0;

  printf("   GOP headers= %4u %s\n", uGOP_ctr, szBuffer);

  uGOP_Code_Prev += uRel_ctr - uGOP_Code_Start;

  if (uPIC_ctr)
  {
    //uTmp8 = uPIC_ctr + uRpt_ctr;
    //if (uRpt_ctr)
    //{
    //  sprintf(szBuffer,"+%4u Rpts =%4u), uRpt_ctr, uTmp8;
    //}
    //else
    //{
    //  szBuffer[0] = 0;
    //}
    printf("   Pic headers= %4u vs %4u Expected GOP  (%us vs %us GOP)\n",
                          uPIC_ctr,      uGOP_Code_Prev,
                         (uPIC_ctr*1001/uFRate1001),
                   (uGOP_Code_Prev*1001/uFRate1001));
  }

  printf("\n\nGOP final span = %d frames\n\n", uRel_ctr);
  GOP_Stats_Inc();

  if (uGOP_ctr)
  {
     printf("GOP FRAMING CATEGORIES:\n");
     printf("    DVD PAL  13-F =%4u\n", uGOPspanDVDPAL);
     printf("        NTSC 17-F =%4u\n", uGOPspanDVDNTSC);
     printf("        DV    1-F =%4u\n", uGOPspan1);
     printf("    MPG General\n");
     printf("        ~Moderate =%4u\n",       uGOPspanMISC);
     printf("            >17 F =%4u Long\n",  uGOPspanLONG);
     printf("            >99 F =%4u HUGE\n",  uGOPspanKDVD);
     printf("            <13 F =%4u Short\n", uGOPspanSHORT );
     printf("           NULL F =%4u EMPTY\n", uGOPspan0);


     cTmp1 = ' ';
     printf("GOP stats:\n");
     for (i = 0; i < 602; i++)
     {
        uTmp1 = uGOP_Frames[i];
        if (uTmp1 > 0)
        {
           if (i == 601)
              cTmp1 = '+';
           printf("   %03d%cframed - %06u GOPS\n", i, cTmp1, uTmp1);
        }
     }
     printf("\n\n");
  }


  int i2;

  printf("\nStart SCR BA =%02us  End scr=%02us  DIFF=%02ds  GAPS=%u\n",
         (uStartTC[256]/90000), (uSCR_Major/90000), ((uSCR_Major-uStartTC[256])/90000),
         uGAP_ctr);

  for (i2 = 0xE0; i2 < 0xEF; i2++)
  {
    if (uStartTC[i2] != 0xFFFFFFFF)
    {
      printf("Start VID %X =%02ds  End VID~%02us  DIFF=%02ds   ADJ=%03dms\n",
             i2, (uStartTC[i2]/90000), (uPTS_Prev[i2]/90000),
                 ((uPTS_Prev[i2]-uStartTC[i2])/90000),
                 uStartADJ);
    }
  }

  //printf("\n");

  for (i2 = 0xC0; i2 < 0xCF; i2++)
  {
    if (uStartTC[i2] != 0xFFFFFFFF)
    {
      printf("Start MPA %X =%02ds  End MPA~%02us  DIFF=%02ds DELAY=%03dms\n",
             i2, (uStartTC[i2]/90000), (uPTS_Prev[i2]/90000),
                                      ((uPTS_Prev[i2]-uStartTC[i2])/90000),
                                       ( int(uStartTC[i2]/90)
                                        -int(uStartTC[0xE0]/90)
                                        +uStartADJ
                                       ));
    }
  }

  //printf("\n");

  for (i2 = 0x80; i2 < 0x8F; i2++)
  {
    if (uStartTC[i2] != 0xFFFFFFFF)
    {
      printf("Start AC3 %X =%02us  End AC3~%02us  DIFF=%02ds  DELAY=%03d\n",
             i2, (uStartTC[i2]/90000), (uPTS_Prev[i2]/90000),
                                      ((uPTS_Prev[i2]-uStartTC[i2])/90000),
                                       (( int(uStartTC[i2]/90)
                                         -int(uStartTC[0xE0]/90)
                                         +uStartADJ
                                        )));
    }
  }


  printf("\n");

  for (i = 0; i < 257; i++)
  {
    if (stream_packets[i])
    {
      // iTmp7 = substream_bytes[i];
      // iTmp8 = iTmp7>>20;  // Megabytes

      switch (i)
      {
        case PROGRAM_STREAM_MAP:
          printf("Program Stream Map packets= %u, total bytes= %u\n",
                             stream_packets[i], stream_bytes[i]);
          break;

        case PRIVATE_STREAM_1:
          printf("Private Stream 1         %6u packets, %8u bytes  %3u MB\n",
                            stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          // if (vob_flag)
          {
            for (substream_id = 0; substream_id < 256; substream_id++)
            {
              if (substream_packets[substream_id])
              {
                iTmp7 = substream_bytes[substream_id];
                iTmp8 = iTmp7>>20;  // Megabytes

                if ((substream_id >= SUBSTREAM_AC3_0)
                 && (substream_id <= SUBSTREAM_AC3_8))
                {
                   strcpy(szBuffer,"AC3 Audio");
                   iTmp6 = substream_id - SUBSTREAM_AC3_0;  // Relative Track Number
                }
                else
                if ((substream_id >= SUBSTREAM_DTS_0)
                 && (substream_id <= SUBSTREAM_DTS_8))
                {
                   strcpy(szBuffer,"DTS Audio");
                   iTmp6 = substream_id - SUBSTREAM_DTS_0;  // Relative Track Number
                }
                else
                if ((substream_id >= SUBSTREAM_DDP_0)
                 && (substream_id <= SUBSTREAM_DDP_F))
                {
                   strcpy(szBuffer,"DD+ Audio");
                   iTmp6 = substream_id - SUBSTREAM_DDP_0;  // Relative Track Number
                }
                else
                if ((substream_id >= SUBSTREAM_PCM_0)
                 && (substream_id <= SUBSTREAM_PCM_F))
                {
                   strcpy(szBuffer,"PCM Audio");
                   iTmp6 = substream_id - SUBSTREAM_PCM_0;  // Relative Track Number
                }
                else
                if ((substream_id >= SUBSTREAM_SUBPIC_0)
                 && (substream_id <= SUBSTREAM_SUBPIC_1F))
                {
                   strcpy(szBuffer,"Subtitle ");
                   iTmp6 = substream_id - SUBSTREAM_SUBPIC_0;  // Relative Track Number
                }
                else
                {
                   strcpy(szBuffer,"Unknown  ");
                   iTmp6 = substream_id;
                }

                printf("     %s Trk %u x%02X %6u packets, %8u bytes  %3u MB   DELAY=%03dms\n",
                           szBuffer, iTmp6, substream_id,
                           substream_packets[substream_id], iTmp7, iTmp8,
                           (( int(uStartTC[substream_id]/90)
                                         -int(uStartTC[0xE0]/90)
                                         +uStartADJ
                                       )));
              }
            }
          }
          break;

        case PRIVATE_STREAM_2:
          printf("Private Stream 2        %6u packets, %8u bytes  %3u MB\n", stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          break;
        case ECM_STREAM:
          printf("ECM Stream packets= %u, total bytes= %u\n", stream_packets[i], stream_bytes[i]);
          break;
        case EMM_STREAM:
          printf("EMM Stream packets= %u, total bytes= %u\n", stream_packets[i], stream_bytes[i]);
          break;
        case PROGRAM_STREAM_DIRECTORY:
          printf("Program Stream Directory packets= %u, total bytes= %u\n", stream_packets[i], stream_bytes[i]);
          break;
        case DSMCC_STREAM:
          printf("DSMCC Stream packets= %u, total bytes= %u\n", stream_packets[i], stream_bytes[i]);
          break;
        case ITUTRECH222TYPEE_STREAM:
          printf("ITU-T Rec. H.222.1 type E packets= %u, total bytes= %u\n", stream_packets[i], stream_bytes[i]);
          break;
        case PADDING_STREAM:
          printf("Padding stream       xBE %7u packets, %8u bytes  %3u MB\n", stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          break;
        case 0xBA:
          printf("PACK Headers         x%02X %7u packets, %8u bytes  %3u MB\n",  i, stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          break;
        case 0xBB:
          printf("SYSTEM Headers       x%02X %7u packets, %8u bytes  %3u MB\n",  i, stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          break;
        case 0xB9:
          printf("END Program Markers  x%02X %7u packets, %8u bytes  %3u MB\n",  i, stream_packets[i], stream_bytes[i], (stream_bytes[i]>>20));
          break;
        case 257:
          printf("ORPHAN Data              %7u packets, %8u bytes  %3u MB\n",
                            stream_packets[i],
                            stream_bytes[i], (stream_bytes[i]>>20));
          break;

        default:
          if ((i >= 0xE0) && (i <= 0xEF))
            printf("MPEG-Video stream %02d x%02X %7u packets, %8u bytes  %3u MB\n",
                        i - 0xE0, i, stream_packets[i],
                        stream_bytes[i],  (stream_bytes[i]>>20));
          else
          if ((i >= 0xC0) && (i <= 0xDF))
            printf("MPEG-Audio stream %02d x%02X %7u packets, %8u bytes  %3u MB\n",
                                  i - 0xC0, i, stream_packets[i],
                                   stream_bytes[i], (stream_bytes[i]>>20));
          else
            printf("Other stream         x%02X %7u packets, %8u bytes  %3u MB\n",
                               i, stream_packets[i],
                               stream_bytes[i], (stream_bytes[i]>>20));
      }
      //printf("                     ................................\n");
    }
  }
  finish_getbits();
  printf("\nbbInfo2 - END                                         .\n");
  return (0);
}


//------------------------------------------------------------------


int PID, packet_data_bytes;

void TS_PES_HDR()
{
  // int PES_packet_length;
  int PTS_DTS_flags, PES_extension_field_length;
  int ESCR_flag, ES_rate_flag, DSM_trick_mode_flag, additional_copy_info_flag;
  int PES_CRC_flag, PES_extension_flag, PES_header_data_length;
  int trick_mode_control, PES_private_data_flag, pack_header_field_flag;
  int program_packet_sequence_counter_flag, PSTD_buffer_flag, PES_extension_flag_2;

                 zprintf(2, "\n  10=%d", getbits(2));
                 zprintf(2, "  PES_scrambling=%d,", getbits(2));
                 zprintf(2, "  PES_priority=%d", get1bit());
                 zprintf(2, "  data_alignment=%d", get1bit());
                 zprintf(2, "  (c)=%d,", get1bit());
                 zprintf(2, " copy=%d\n", get1bit());
                 PTS_DTS_flags = getbits(2);
                 zprintf(2, "  FLAGS: PTS_DTS_f=%d", PTS_DTS_flags);
                 ESCR_flag = get1bit();
                 zprintf(2, "  ESCR_f=%d", ESCR_flag);
                 ES_rate_flag = get1bit();
                 zprintf(2, "  ES_rate_f=%d", ES_rate_flag);
                 DSM_trick_mode_flag = get1bit();
                 zprintf(2, "  DSM_trick_mode_f=%d", DSM_trick_mode_flag);
                 additional_copy_info_flag = get1bit();
                 zprintf(2, "  addit_copy_f=%d\n", additional_copy_info_flag);
                 PES_CRC_flag = get1bit();
                 zprintf(2, "  PES_CRC_flag= %d,", PES_CRC_flag);
                 PES_extension_flag = get1bit();
                 zprintf(2, "  PES_extension_flag= %d,", PES_extension_flag);
                 PES_header_data_length = getbits(8);
                 zprintf(2, "  PES_HEADER_data_length= %d\n", PES_header_data_length);
                 packet_data_bytes -= (3 + PES_header_data_length);

                 //if ((verbose_level == 1) && (PTS_DTS_flags != 2) && (PTS_DTS_flags != 3))
                 //  printf("\n");
                 if (PTS_DTS_flags == 2)
                 {
                   if (verbose_level == 1)
                   {
                     getbits(4);
                     uPTS_Major = getbits(3);
                     if (uPTS_Major & 0x00000004)
                       dPts_value = 4294967296.0;
                     else
                       dPts_value = 0.0;
                     uPTS_Major = uPTS_Major << 30;
                     get1bit();
                     uPTS_Major |= (getbits(15) << 15);
                     get1bit();
                     uPTS_Major |= getbits(15);
                     dPts_value += (long double) uPTS_Major;
                     get1bit();

                     printf("PTS= %08Lf ms\n", dPts_value / 90.0);
                   }
                   else
                   {
                     if (verbose_level > 3)
                       strcat(level3, "PTS");
                     else
                     //if (verbose_level == 2)
                     //  printf("  PTS fields\n");

                     zprintf(2, "\n  PTS 0010=%d", getbits(4));
                     zprintf(2, "  [32..30]=%5d,", getbits(3));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [29..15]=%5d,", getbits(15));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [14..00]=%5d,", getbits(15));
                     zprintf(2, "  m%d\n", get1bit());
                   }
                   bytes_used += 5;
                 }

                 if (PTS_DTS_flags == 3)
                 {
                   if (verbose_level == 1)
                   {
                     getbits(4);
                     uPTS_Major = getbits(3);
                     if (uPTS_Major & 0x00000004)
                       dPts_value = 4294967296.0;
                     else
                       dPts_value = 0.0;
                     uPTS_Major = uPTS_Major << 30;
                     get1bit();
                     uPTS_Major |= (getbits(15) << 15);
                     get1bit();
                     uPTS_Major |= getbits(15);
                     dPts_value += (long double) uPTS_Major;
                     get1bit();
                     printf("PTS= %08Lf ms, ", dPts_value / 90.0);
                     getbits(4);
                     uDTS_Major = getbits(3);
                     if (uDTS_Major & 0x00000004)
                       dDts_value = 4294967296.0;
                     else
                       dDts_value = 0.0;
                     uDTS_Major = uDTS_Major << 30;
                     get1bit();
                     uDTS_Major |= (getbits(15) << 15);
                     get1bit();
                     uDTS_Major |= getbits(15);
                     dDts_value += (long double) uDTS_Major;
                     get1bit();
                     printf("DTS= %08Lf ms\n", dDts_value / 90.0);
                   }
                   else
                   {
                     //if (verbose_level == 2)
                     //  printf("  PTS");
                     zprintf(2, "\n  PTS 0011=%d", getbits(4));
                     zprintf(2, "  [32..30]=%5d,", getbits(3));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [29..15]=%5d,", getbits(15));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [14..00]=%5d,", getbits(15));
                     zprintf(2, "  m%d\n", get1bit());
                     //if (verbose_level == 2)
                     //  printf("  DTS fields");
                     zprintf(2, "  DTS 0001=%d", getbits(4));
                     zprintf(2, "  [32..30]=%5d,", getbits(3));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [29..15]=%5d,", getbits(15));
                     zprintf(2, "  m%d", get1bit());
                     zprintf(2, "  [14..00]=%5d,", getbits(15));
                     zprintf(2, "  m%d\n\n", get1bit());
                     if (verbose_level > 3)
                       strcat(level3, "PTSDTS");
                   }
                   bytes_used += 10;
                 }

                 if (ESCR_flag == 1)
                 {
                   if (verbose_level == 2)
                     printf("  ESCR fields");
                   zprintf(2, "    res= %d", getbits(2));
                   zprintf(2, "  ESCR_base[32..30]= %5d,", getbits(3));
                   zprintf(2, "  m%d", get1bit());
                   zprintf(2, "  ESCR_base[29..15]= %5d,", getbits(15));
                   zprintf(2, "  m%d", get1bit());
                   zprintf(2, "  ESCR_base[14..00]= %5d,", getbits(15));
                   zprintf(2, "  m%d", get1bit());
                   zprintf(2, "  ESCR_extension= %d,", getbits(9));
                   zprintf(2, "  m%d\n", get1bit());
                   bytes_used += 6;
                 }

                 if (ES_rate_flag == 1)
                 {
                   if (verbose_level == 2)
                     printf("  ES rate fields:");
                   zprintf(2, "    m%d\n", get1bit());
                   zprintf(2, "    ES_rate= %d,", getbits(22));
                   zprintf(2, "    m%d\n", get1bit());
                   bytes_used += 3;
                 }

                 if (DSM_trick_mode_flag == 1)
                 {
                   trick_mode_control = getbits(3);
                   if (verbose_level == 2)
                     printf("  DSM trick mode fields:");
                   zprintf(2, "    trick_mode_control= %d\n", trick_mode_control);
                   switch (trick_mode_control)
                   {
                     case FAST_FORWARD:
                       if (verbose_level == 2)
                         printf("    fast forward fields:");
                       zprintf(2, "      field_id= %d,", getbits(2));
                       zprintf(2, "      intra_slice_refresh= %d,", get1bit());
                       zprintf(2, "      frequency_truncation= %d\n", getbits(2));
                       break;

                     case SLOW_MOTION:
                       if (verbose_level == 2)
                         printf("    slow motion fields:");
                       zprintf(2, "      rep_cntrl= %d\n", getbits(5));
                       break;

                     case FREEZE_FRAME:
                       if (verbose_level == 2)
                         printf("    freeze frame fields:");
                       zprintf(2, "      field_id= %d,", getbits(2));
                       zprintf(2, "      reserved= %d\n", getbits(3));
                       break;

                     case FAST_REVERSE:
                       if (verbose_level == 2)
                         printf("    fast reverse fields:");
                       zprintf(2, "      field_id= %d,", getbits(2));
                       zprintf(2, "      intra_slice_refresh= %d,", get1bit());
                       zprintf(2, "      frequency_truncation= %d\n", getbits(2));
                       break;

                     case SLOW_REVERSE:
                       if (verbose_level == 2)
                         printf("    slow reverse fields:");
                       zprintf(2, "      rep_cntrl= %d\n", getbits(5));
                       break;

                     default:
                       if (verbose_level == 2)
                         printf("    reserve fields:");
                       zprintf(2, "      reserved= %d\n", getbits(5));
                       break;
                   }
                   bytes_used++;
                 }

                 if (additional_copy_info_flag == 1)
                 {
                   if (verbose_level == 2)
                     printf("  additional copy info fields:");
                   zprintf(2, "    mk=%d\n", get1bit());
                   zprintf(2, "    additional_copy_info= %d\n", getbits(7));
                   bytes_used++;
                 }

                 if (PES_CRC_flag == 1)
                 {
                   if (verbose_level == 2)
                     printf("  PES CRC field:");
                   zprintf(2, "    previous_PES_packet_CRC= %d\n", getbits(16));
                   bytes_used += 2;
                 }

                 if (PES_extension_flag == 1)
                 {
                   if (verbose_level == 2)
                     printf("  PES extension fields\n");
                   PES_private_data_flag = get1bit();
                   zprintf(2, "    PES_private_data_flag= %d\n", PES_private_data_flag);
                   pack_header_field_flag = get1bit();
                   zprintf(2, "    pack_header_field_flag= %d\n", pack_header_field_flag);
                   program_packet_sequence_counter_flag = get1bit();
                   zprintf(2, "    program_packet_sequence_counter_flag= %d\n", program_packet_sequence_counter_flag);
                   PSTD_buffer_flag = get1bit();
                   zprintf(2, "    P-STD_buffer_flag= %d,", PSTD_buffer_flag);
                   zprintf(2, "    reserved= %d\n", getbits(3));
                   PES_extension_flag_2 = get1bit();
                   zprintf(2, "    PES_extension_flag_2= %d\n", PES_extension_flag_2);
                   bytes_used++;

                   if (PES_private_data_flag == 1)
                   {
                     if (verbose_level == 2)
                       printf("    PES private data fields\n");
                     for (iPayload = 0; iPayload < 128; iPayload++)
                       getbits(8);
                     zprintf(2, "      read %d bytes of private data\n", iPayload);
                     bytes_used += 16;
                   }

                   if (pack_header_field_flag == 1)
                   {
                     printf("\n    ****\n    ****    pack header field flag value not allowed in program streams\n    ****\n");
                     //exit(1);
                   }

                   if (program_packet_sequence_counter_flag == 1)
                   {
                     if (verbose_level == 2)
                       printf("    program packet sequence counter fields:");
                     zprintf(2, "      mk=%d\n", get1bit());
                     zprintf(2, "      program_packet_sequence_counter= %d,", getbits(7));
                     zprintf(2, "      mk=%d", get1bit());
                     zprintf(2, "      MPEG1_MPEG2_identifier= %d,", get1bit());
                     zprintf(2, "      original_stuff_length= %d\n", getbits(6));
                     bytes_used += 2;
                   }

                   if (PSTD_buffer_flag == 1)
                   {
                     if (verbose_level == 2)
                       printf("    P-STD buffer fields:");
                     zprintf(2, "      01= %d", getbits(2));
                     zprintf(2, "      P-STD_buffer_scale= %d,", get1bit());
                     zprintf(2, "      P-STD_buffer_size= %d\n", getbits(13));
                     bytes_used += 2;
                     if (verbose_level > 3)
                       strcat(level3, " P-STD");
                   }

                   if (PES_extension_flag_2 == 1)
                   {
                     if (verbose_level == 2)
                       printf("    PES extension 2 fields:");
                     zprintf(2, "      mk=%d", get1bit());
                     PES_extension_field_length = getbits(7);
                     zprintf(2, "      PES_extension_field_length= %d\n", PES_extension_field_length);
                     bytes_used++;
                     for (iPayload = 0; iPayload < PES_extension_field_length; iPayload++)
                     {
                       getbits(8);
                       bytes_used++;
                     }
                     zprintf(2, "      read %d bytes of PES extension field data\n", iPayload);
                   }
                 }
                 for (iPayload = 0; 
                      iPayload < PES_header_data_length - bytes_used; 
                      iPayload++)
                   getbits(8);

                 if (iPayload)
                   zprintf(2, "  read %d stuffing bytes\n", iPayload);
}

//-------


struct stream_entry {
  unsigned long stream_id;
  unsigned long substream_id;
  unsigned long stream_packets;
  unsigned long stream_bytes;
  unsigned long substream_packets;
  unsigned long substream_bytes;
  int video_sync;
  int pcm_bytes;
  int audio_sync;
  int subpic_bytes;
  int ac3_bytes;
  int dts_bytes;
};
stream_entry *streams[8192];


void TS_Stream_id()
{
          zprintf(0, "  Stream_id= %02X  ", streams[PID]->stream_id);

          switch (streams[PID]->stream_id)
          {
            case PROGRAM_STREAM_MAP:
              printf("Program Stream Map, ");
              break;
            case PRIVATE_STREAM_1:
              printf("Private Stream 1, ");
              break;
            case PRIVATE_STREAM_2:
              printf("Private Stream 2, ");
              break;
            case ECM_STREAM:
              printf("ECM Stream, ");
              break;
            case EMM_STREAM:
              printf("EMM Stream, ");
              break;
            case PROGRAM_STREAM_DIRECTORY:
              printf("Program Stream Directory, ");
              break;
            case DSMCC_STREAM:
              printf("DSMCC Stream, ");
              break;
            case ITUTRECH222TYPEE_STREAM:
              printf("ITU-T Rec. H.222.1 type E, ");
              break;
            case PADDING_STREAM:
              printf("Padding Stream, ");
              break;
            default:
              if ((streams[PID]->stream_id >= 0xE0) 
               && (streams[PID]->stream_id <= 0xEF))
              {
                zprintf(0, "Video Stream %d, ", (streams[PID]->stream_id - 0xE0));
              }
              else
              if ((streams[PID]->stream_id >= 0xC0) 
               && (streams[PID]->stream_id <= 0xDF))
              {
                  zprintf(0, "MPEG Audio Stream %d, ", (streams[PID]->stream_id - 0xC0));
              }
              else
              if (streams[PID]->stream_id == 0)
              {
                 switch (PID)
                 {
                    case PROGRAM_ASSOCIATION_TABLE:
                      printf("  PAT - Program Association Table, ");
                      break;
                    case CONDITIONAL_ACCESS_TABLE:
                      printf("  CAT - Conditional Access Table, ");
                      break;
                    case NULL_PACKET:
                      printf("  NULL packet, ");
                      break;
                    default:
                      printf("  Other packet, ");
                      break;
                 } // end SWITCH PID
              } // zero stream_id
              else
                 printf(" Unknown Stream-id ");
          }

}



void TS_VID_Data()
{
  int k;
  iPayload = 0;
  while (iPayload < packet_data_bytes)
  {
    k = getbits(8);
    switch (streams[PID]->video_sync)
    {
      case 0:
        if (k == 0x00)
          streams[PID]->video_sync = 1;
        break;
      case 1:
        if (k == 0x00)
          streams[PID]->video_sync = 2;
        else
          streams[PID]->video_sync = 0;
        break;
      case 2:
        if (k != 0x01)
        {
          if (k != 0x00)
            streams[PID]->video_sync = 0;
        }
        else
        {
          offset = iPayload - 2;
          switch (look_ahead(8))
          {
            case USER_DATA_START_CODE:
              printf("  * %03d - user_data_start_code\n", offset);
              break;

            case SEQUENCE_HEADER_CODE:
              if (offset)
              {
                strcpy(szFlag,"UNALIGNED");
                iSeq_Unaligned_ctr++;
              }
              else
                 szFlag[0] = 0x00;

              printf("  * %03d - sequence_header_code     === SEQ %s\n",
                       offset, szFlag);
              iSeq_ctr++;
              break;

            case EXTENSION_START_CODE:
              printf("  * %03d - extension_start_code\n", offset);
              break;

            case SEQUENCE_END_CODE:
              printf("  * %03d - sequence_end_code\n", offset);
              break;

            case GROUP_START_CODE:
              // HOW MANY FRAMES IN Previous GOP
              if (uRel_ctr == 13) uGOPspanDVDPAL++;  else
              if (uRel_ctr == 17) uGOPspanDVDNTSC++; else
              if (uRel_ctr ==  1) uGOPspan1++;       else
              if (uRel_ctr ==  0) uGOPspan0++;       else
              if (uRel_ctr  > 99) uGOPspanKDVD++;    else
              if (uRel_ctr  > 18) uGOPspanLONG++;    else
              if (uRel_ctr  < 12) uGOPspanSHORT++;   else
              uGOPspanMISC++;

              printf("  * %03d - group_of_pictures header *** GOP   PrevSpan=%u\n",
                            offset,       uRel_ctr);
              uGOP_ctr++;
              uRel_ctr = 0;
              break;

            case PICTURE_START_CODE:
              printf("  * %03d - picture_start_code       --- PIC\n", offset);
              uPIC_ctr++;
              uRel_ctr++;
              break;
          }
          streams[PID]->video_sync = 0;
        }
    }
    iPayload++;
  }
}





void TS_MPA_Data()
{
  int uMPA_Layer_ix, BitRate_ix, Freq_ix, padding_bit;
  unsigned int uMPA_25_LSF, id;

  int k; // , id;
  for (iPayload = 0; iPayload < packet_data_bytes; iPayload++)
  {
    k = getbits(8);
    if (!iPayload)
        printf("    00 = 0x%02X\n", k);

    switch (streams[PID]->audio_sync)
    {
      case -1:
        if (k == 0xFF)
          streams[PID]->audio_sync = -2;
        break;

      case -2:
        if ((k & 0xF0) == 0xF0)
        {
          offset = iPayload - 1;
          printf("    %d - audio syncword\n", offset);

          id           =    (k & 0x08) >> 3; // id is opposite of LSF
          uMPA_25_LSF  = 3-((k>>3)&3); // Mpeg2.5 hijacks low-order bit of syncword
          if (uMPA_25_LSF > 1)
              uMPA_Mpeg25 = 1;
          else
              uMPA_Mpeg25 = 0;

          if (id)   // LowSamplingFrequency Extension uses the old "ID" Flag
              uMPA_LSF    = 0;
          else
              uMPA_LSF    = 1;

          uMPA_Layer_ix    = (k & 0x06) >> 1;
          streams[PID]->audio_sync = -3;
        }
        else
          streams[PID]->audio_sync = -1;
        break;

      case -3:
        BitRate_ix  = (k & 0xF0) >> 4;
        Freq_ix     = (k & 0x0C) >> 2;
        padding_bit = (k & 0x02) >> 1;
        uMPA_kBitRate = // BitRate_tbl[3 - uMPA_Layer_ix][BitRate_ix];
                        MPA_KBIT_RATE[uMPA_LSF]
                                     [3 - uMPA_Layer_ix][BitRate_ix];
        streams[PID]->audio_sync = ((uMPA_kBitRate * 1000 
                                 / MPA_SAMPLE_HZ[uMPA_25_LSF][Freq_ix])  // uFrequency[Freq_ix]) 
                                 * slots[uMPA_Layer_ix]) - 4;
        if (padding_bit)
            streams[PID]->audio_sync += 1;
        break;

      default:
        streams[PID]->audio_sync -= 1;
    }
  }
}


/* TS PS1 does NOT have substreams
void TS_DD_HDR()
{
  int iTST;

  iTST = look_ahead(8);

  if ((streams[PID]->substream_id >= SUBSTREAM_AC3_0)
   && (streams[PID]->substream_id <= SUBSTREAM_AC3_8))
  {
     zprintf(1, "AC3 Audio Trk#%d, ", streams[PID]->substream_id - SUBSTREAM_AC3_0);
     if (iTST != 0x77)
     {
        zprintf(1, "#syncwords= 0x%02X, ", getbits(8));
        zprintf(1, "1st offset= 0x%04X\n", getbits(16));
        packet_data_bytes -= 3;
     }
  }
  else
  if ((streams[PID]->substream_id >= SUBSTREAM_DTS_0)
   && (streams[PID]->substream_id <= SUBSTREAM_DTS_8))
  {
     zprintf(1, "DTS Audio Trk#%d\n", streams[PID]->substream_id - SUBSTREAM_DTS_0);
     if (iTST != 0x77)
     {
        zprintf(1, "#syncwords= 0x%02X, ", getbits(8));
        zprintf(1, "1st offset= 0x%04X\n", getbits(16));
        packet_data_bytes -= 3;
     }
  }
  else
  if ((streams[PID]->substream_id >= SUBSTREAM_DDP_0)
   && (streams[PID]->substream_id <= SUBSTREAM_DDP_F))
  {
     zprintf(1, "DD+ Audio Trk#%d, ", streams[PID]->substream_id - SUBSTREAM_DDP_0);
     if (iTST != 0x77)
     {
        zprintf(1, "#syncwords= 0x%02X, ", getbits(8));
        zprintf(1, "1st offset= 0x%04X\n", getbits(16));
        packet_data_bytes -= 3;
     }
  }
  else
  if ((streams[PID]->substream_id >= SUBSTREAM_PCM_0)
   && (streams[PID]->substream_id <= SUBSTREAM_PCM_F))
  {
     zprintf(1, "PCM Audio Trk#%d\n", streams[PID]->substream_id - SUBSTREAM_PCM_0);
  }
  else
  if ((streams[PID]->substream_id >= SUBSTREAM_SUBPIC_0)
   && (streams[PID]->substream_id <= SUBSTREAM_SUBPIC_1F))
  {
      zprintf(1, "SUBtitle Trk#%d\n", streams[PID]->substream_id - SUBSTREAM_SUBPIC_0);
  }
  else
  {
      zprintf(1, "  UNK substream= 0x%02X", streams[PID]->substream_id);
      if (iTST == 0x77)
      {
        zprintf(1, "MAYBE AC3 Audio Trk#%d, ", streams[PID]->substream_id);
      }
  }

}
*/



void transport_stream()
{
  int uPackSize;
  int i, k,  transport_packets;
  int firsttime;
  
  int transport_error_indicator, payload_unit_start_indicator;
  int transport_priority, transport_scrambling_control;
  int adaptation_field_control, continuity_counter;
  int adaptation_field_length, adaptation_bytes;
  int PCR_flag, OPCR_flag, splicing_point_flag, transport_private_data_flag;
  int adaptation_field_extension_flag, adaptation_field_extension_length;
  int ltw_flag, piecewise_rate_flag, seamless_splice_flag;

  char tmpStr[256];
  int iTmp1, iTmp2;
  double PrevPack_bit_count;

  unsigned int uTmp1, uTmp2, uTmp6, uTmp7;

  for (i = 0; i < 8192; i++)
    streams[i] = NULL;

  firsttime = 0;
  transport_packets = 0;
  PrevPack_bit_count = 0.0;

  do
  {
    i = getbits(8);



/*
next1pass:
    if (firsttime)
    {
      if (i != 0x47) // PACK_START_CODE)
      {
        i = getbits(8);
        goto next1pass;
      }
      firsttime = false;
    }
*/


Resync47:
    if (i != 0x47)
    {
      /*
          printf("\n\nAttempting ReSynch ... ");
          if (seek_sync(0x47, 8))
          {
            i = 0x47;
            printf("       .... succesfully resynched \n\n");
          }
          else
          {
            printf("\nRESYNC FAILED - EOF");
            exit(1);
          }
       */
            uResync_Ctr++;
            stream_packets[257]++;

            uFromB =  (unsigned int)(bitcount() / 8 - 4);
            uTmp6 = uFromB/1024;
            uTmp7 = uTmp6 - uErrPrevK;
            printf("\n\n*BAD* Unknown bits:  x%06X   at RBA %d [x%06X]  Since=%uK\n",
                                          i, uFromB, uFromB, uTmp7);
            printf("\nAttempting ReSynch ... ");
            uFromKB = uFromB / 1024;
            uPrevKB = 0;
            uErrPrevK = uTmp6;
Sync_More:
            if (seek_sync(0x47, 8))
            {
                i = 0x47;
                if (i >= 0x0100 && i <= 0x01B8) // Picture control
                {
                  uSkipKB = (unsigned int)(bitcount() / 8192) - uFromKB;
                  if ((uSkipKB - uPrevKB) >= 1024)
                  {
                    printf("\n   Skipped %d megabytes...",
                                 (uSkipKB/1024));
                    uPrevKB = uSkipKB;
                  }
                  goto Sync_More;               // Not a pkt hdr
                }

                uTmp1 =  (unsigned int)(bitcount() / 8 - 4);
                uTmp2 = uTmp1 - uFromB;
                printf("Succesfull Resync after %d bytes\n\n",
                                                          uTmp2);
                stream_bytes[257] += uTmp2;
            }
    }
    if (PrevPack_bit_count == 0.0)
        uPackSize = 0;
    else
        uPackSize = ((unsigned long)(bitcount() - PrevPack_bit_count)) >> 3;
    PrevPack_bit_count = bitcount();

    if (uPackSize)
    {
        zprintf(1, "  PktSize=%d ", uPackSize );
        zprintf(1, "  =x%02X\n",     uPackSize );
    }

    transport_packets++;
    zprintf(1, "\nPKT#%d, ", transport_packets - 1);
    zprintf(1, "sync=%02X", i);
    iTmp1 = (PrevPack_bit_count/8.0) - 1;
    zprintf(1, "   LOC=x%04X     ", iTmp1);

   transport_error_indicator = get1bit();
    zprintf(2, "  err=%d  ", transport_error_indicator);
   payload_unit_start_indicator = get1bit();
   transport_priority = get1bit();
    zprintf(2, "  pri=%d\n", transport_priority);
   PID = getbits(13);

    if (streams[PID] == NULL)
    {
      streams[PID] = (stream_entry*)malloc(sizeof(stream_entry));
      if (streams[PID] == NULL)
      {
        printf("\n    ****\n    **** Unable to allocate memory for stream entry\n    ****.\n");
        exit(1);
      }
      streams[PID]->stream_id = 0;
      streams[PID]->substream_id = 0;
      streams[PID]->stream_packets = 0;
      streams[PID]->stream_bytes = 0;
      streams[PID]->substream_packets = 0;
      streams[PID]->substream_bytes = 0;
      streams[PID]->video_sync = 0;
      streams[PID]->pcm_bytes = 0;
      streams[PID]->audio_sync = -1;
      streams[PID]->subpic_bytes = 0;
      streams[PID]->ac3_bytes = 0;
      streams[PID]->dts_bytes = 0;
    }

    zprintf(2, "  PID= x%04X", PID);
    zprintf(2, " =%03d", PID);
    if (payload_unit_start_indicator)
        zprintf(2, "   %d=payload_unit_start  ", payload_unit_start_indicator);
    //else
    //    zprintf(2, "   %d=pus  ", payload_unit_start_indicator);
    transport_scrambling_control = getbits(2);
    if (transport_scrambling_control)
       zprintf(2, "  scrambling= %d\n", transport_scrambling_control);

    iTmp2  = getbits(1);
    iTmp1  = getbits(1);
    adaptation_field_control = iTmp2 * 2 + iTmp1;
    zprintf(2, "   Adapt: Ctl=%d,", adaptation_field_control);
    zprintf(2, "Present=%d ", iTmp2);
    zprintf(2, " Payload=%d ", iTmp1);
   continuity_counter = getbits(4);
    zprintf(2, " CCtr=%d\n", continuity_counter);

   if (adaptation_field_control > 1)
    {
      adaptation_field_length = getbits(8);
      adaptation_bytes = 0;
      zprintf(2, "   AdaptLen=%d\n", adaptation_field_length);
      if (adaptation_field_length)
      {
        zprintf(2, "    Discont=%d", get1bit());
        zprintf(2, " Random=%d", get1bit());
        zprintf(2, " EPri=%d\n", get1bit()); // elementary_stream_priority_indicator
        PCR_flag = get1bit();
        zprintf(2, "    PCR_flag=%d", PCR_flag);
        OPCR_flag = get1bit();
        zprintf(2, "    OPCR_flag=%d", OPCR_flag);
        splicing_point_flag = get1bit();
        zprintf(2, "    SplicePoint=%d\n", splicing_point_flag);
        transport_private_data_flag = get1bit();
        zprintf(2, "    private=%d", transport_private_data_flag);
        adaptation_field_extension_flag = get1bit();
        zprintf(2, "    AdaptXtn=%d\n", adaptation_field_extension_flag);
        adaptation_bytes++;

      if (PCR_flag)
        {
          if (verbose_level == 2)
            printf("    PCR fields:");
          zprintf(2, "  PCR[32..30]=%05d", getbits(3));
          zprintf(2, "  [29..15]=%05d", getbits(15));
          zprintf(2, "  [14..00]=%05d", getbits(15));
       zprintf(2, "  0x3F= %02X", getbits(6));
       zprintf(2, "  PCR_extension= %d\n", getbits(9));
          adaptation_bytes += 6;
      }

      if (OPCR_flag)
        {
          if (verbose_level == 2)
            printf("    OPCR fields:");
          zprintf(2, "  OPCR[32..30]=%05d", getbits(3));
          zprintf(2, "  [29..15]=%05d", getbits(15));
          zprintf(2, "  [14..00]=%05d", getbits(15));
       zprintf(2, "  0x3F=%02X", getbits(6));
       zprintf(2, "  OPCR_extension=%d\n", getbits(9));
          adaptation_bytes += 6;
      }

      if (splicing_point_flag)
        {
        zprintf(2, "    splice_countdown= %d\n", getbits(8));
          adaptation_bytes++;
        }

        if (transport_private_data_flag)
        {
       iPayload = getbits(8);
          zprintf(2, "    transport_private_data_length= %d\n", iPayload);
       for (k = 0; k < iPayload; k++)
        getbits(8);
          zprintf(2, "      read %d private_data_bytes\n", iPayload);
          adaptation_bytes += (iPayload + 1);
      }

      if (adaptation_field_extension_flag)
        {
          adaptation_field_extension_length = getbits(8);
          adaptation_bytes += (adaptation_field_extension_length + 1);
          i = 1;
        zprintf(2, "    adaptation_field_extension_length= %d\n", adaptation_field_extension_length);
        ltw_flag = get1bit();
          zprintf(2, "      ltw_flag= %d", ltw_flag);
        piecewise_rate_flag = get1bit();
          zprintf(2, "      piecewise_rate_flag = %d", piecewise_rate_flag);
        seamless_splice_flag = get1bit();
          zprintf(2, "      seamless_splice_flag= %d", seamless_splice_flag);
        zprintf(2, "      0x1F= %02X\n", getbits(5));
          adaptation_bytes += (adaptation_field_extension_length + 1);

        if (ltw_flag)
          {
          zprintf(2, "      ltw_valid_flag  = %d", get1bit());
          zprintf(2, "      ltw_offset= %d\n", getbits(15));
            i += 2;
        }

        if (piecewise_rate_flag)
          {
        zprintf(2, "      0x3= %X", getbits(2));
        zprintf(2, "      piecewise_rate= %d\n", getbits(22));
            i += 3;
        }

        if (seamless_splice_flag)
          {
          zprintf(2, "      splice_type= %d", getbits(4));
          zprintf(2, "  DTS_next_AU[32..30]= %5d,", getbits(3));
          zprintf(2, "  m%d", get1bit());
          zprintf(2, "  DTS_next_AU[29..15]= %5d,", getbits(15));
          zprintf(2, "  m%d\n", get1bit());
          zprintf(2, "  DTS_next_AU[14..00]= %5d,", getbits(15));
          zprintf(2, "  m%d\n", get1bit());
            i += 5;
        }

          k = adaptation_field_extension_length - i;
          if (k > 0)
          {
          for (iPayload = 0; iPayload < k; iPayload++)
            getbits(8);
            zprintf(2, "      read %d stuffing bytes\n", iPayload);
          }
        }
    }
      k = adaptation_field_length - adaptation_bytes;
      if (k > 0)
      {
      for (iPayload = 0; iPayload < k; iPayload++)
        getbits(8);
        zprintf(2, "    read %d stuffing bytes\n", iPayload);
      }
      adaptation_bytes = (adaptation_field_length + 1); // account for length field
   }
    else
    {
      adaptation_bytes = 0;
      if (adaptation_field_control == 0)
      {
        zprintf(2, "    WARNING - NON-STD ADAPTATION CONTROL - x%02X\n",
                                         adaptation_field_control);
        i = getbits(8);
        if (i != 0x47)
        {
          for (iPayload = 0; iPayload < 184; iPayload++)
            i = getbits(8);
            zprintf(2, "       read %d PRESUMED data bytes\n", iPayload);
        }
        goto Resync47;
      }
    }





   if ((adaptation_field_control == 1) || (adaptation_field_control == 3))
    {

      iFirstPktOfSubStream = (!(streams[PID]->stream_packets++)) && !verbose_level;
      packet_data_bytes = 184 - adaptation_bytes;
      zprintf(2, "  PAYLOAD length= %d\n", packet_data_bytes);

      if (packet_data_bytes > 0)
      {
        if ((look_ahead(24) == 0x000001) 
         && payload_unit_start_indicator && (PID > 0x0F))
        {
           i = getbits(32);
           packet_data_bytes -= 4;
           if ((i >> 8) != PACKET_START_CODE_PREFIX)
           {
             printf("\n    ****\n    ****Packet start code not found.\n    ****\n");
             exit(1);
           }
           streams[PID]->stream_id = i & 0x000000FF;

           TS_Stream_id();

           //if (verbose_level == 1)
             zprintf(1, " Header pkt#%d,  ", streams[PID]->stream_packets - 1);
           //else
           //  zprintf(1, "pkt#%d\n", streams[PID]->stream_packets - 1);
           PES_packet_length = getbits(16);
           packet_data_bytes -= 2;
           zprintf(2, "  PES_packet_length= %d", PES_packet_length);
           zprintf(2, " x%04X\n", PES_packet_length);
           
           switch (streams[PID]->stream_id)
           {
             case PROGRAM_STREAM_MAP:
             case ECM_STREAM:
             case EMM_STREAM:
             case PROGRAM_STREAM_DIRECTORY:
             case DSMCC_STREAM:
             case ITUTRECH222TYPEE_STREAM:
               for (iPayload = 0; iPayload < packet_data_bytes; iPayload++)
                    getbits(8);
               if (verbose_level > 2)
                   strcat(level3, "o");
               zprintf(2, "  read %d PES packet data bytes\n", iPayload);
               streams[PID]->stream_bytes += iPayload;
               break;

             case PADDING_STREAM:
             
               iPayload = 0;
               iRemain = packet_data_bytes;
               //if (verbose_level == 2) //   || iFirstPktOfSubStream)
               //    Hex_Prt(32);

               for (; iPayload < packet_data_bytes; iPayload++)
                 getbits(8);
               if (verbose_level > 2)
                 strcat(level3, "p");
               zprintf(2, "  read %d padding_bytes\n", iPayload);
               streams[PID]->stream_bytes += iPayload;
               break;

             default:
               bytes_used = 0;
               TS_PES_HDR();

               if ((streams[PID]->stream_id == PRIVATE_STREAM_1)) // && (vob_flag))
               {
                    //streams[PID]->substream_id = getbits(8);
                    //zprintf(1, "  Substream= 0x%02X  ", streams[PID]->substream_id);
                    //packet_data_bytes--;
                    //TS_DD_HDR();
                    iTmp1 = look_ahead(16);
                    if (iTmp1 == 0x0B77)
                        zprintf(1, "  AC3 Audio Trk#%d, ", 0);

               } // END-IF Private Stream 1
     
               break;
           }
        }
        else
        {
          TS_Stream_id();

          zprintf(1, "continuation pkt#%d\n", streams[PID]->stream_packets - 1);
        }

        // process remaining data in TS packet

        // non detailed report
        if (verbose_level != 2)
        {

          for (iPayload = 0; iPayload < packet_data_bytes; iPayload++)
               getbits(8);
          iPayload++;
        } // END Verbose Level NOT=2

        else
        { // detailed report
          if ((streams[PID]->stream_id >= VIDEO_STREAM_0) 
           && (streams[PID]->stream_id <= VIDEO_STREAM_F))
          {
              i = streams[PID]->stream_id - VIDEO_STREAM_0;
              TS_VID_Data();
          } // END-IF Video Stream
          else
          if ((streams[PID]->stream_id >= AUDIO_STREAM_0) 
           && (streams[PID]->stream_id <= AUDIO_STREAM_1F))
          {
              i = streams[PID]->stream_id - AUDIO_STREAM_0;
              TS_MPA_Data();
          } // END-IF AudioStream
          else
          {
              iPayload = 0;
              if (streams[PID]->stream_id == PRIVATE_STREAM_1)
              {
                 iRemain = packet_data_bytes;
                 if (packet_data_bytes > 0)
                     Hex_Prt(32);
              }
              for (; iPayload < packet_data_bytes; iPayload++)
                   getbits(8);
          }
        } // END-IF Verbose Level 2

        // finished packet - roll totals
        if (iPayload)
        {
          zprintf(2, "  read %d TS packet data bytes\n", iPayload);
          streams[PID]->stream_bytes += iPayload;
          if ((streams[PID]->stream_id == PRIVATE_STREAM_1)) // && (vob_flag))
          {
            streams[PID]->substream_packets++;
            streams[PID]->substream_bytes += iPayload;
            if ((streams[PID]->substream_id >= SUBSTREAM_AC3_0) && (streams[PID]->substream_id <= SUBSTREAM_AC3_8))
              streams[PID]->ac3_bytes += iPayload - 4;
            if ((streams[PID]->substream_id >= SUBSTREAM_DTS_0) && (streams[PID]->substream_id <= SUBSTREAM_DTS_8))
              streams[PID]->dts_bytes += iPayload - 4;
            if ((streams[PID]->substream_id >= SUBSTREAM_PCM_0) && (streams[PID]->substream_id <= SUBSTREAM_PCM_F))
              streams[PID]->pcm_bytes += iPayload - 7;
            if ((streams[PID]->substream_id >= SUBSTREAM_SUBPIC_0) && (streams[PID]->substream_id <= SUBSTREAM_SUBPIC_1F))
              streams[PID]->subpic_bytes += iPayload - 1;
          }
        }

        if (verbose_level > 2)
        {
            if ((streams[PID]->stream_id >= 0xE0) && (streams[PID]->stream_id <= 0xEF))
            {
              strcat(level3, "v");
              sprintf(tmpStr, "%d", streams[PID]->stream_id - 0xE0);
              strcat(level3, tmpStr);
            }
            else
            if ((streams[PID]->stream_id >= 0xC0) && (streams[PID]->stream_id <= 0xDF))
            {
              strcat(level3, "a");
              sprintf(tmpStr, "%d", streams[PID]->stream_id - 0xC0);
              strcat(level3, tmpStr);
            }
            else
            if (streams[PID]->stream_id == PRIVATE_STREAM_1)
            {
                strcat(level3, "1");
                // if (vob_flag)
                {
                  sprintf(tmpStr, "%02X", streams[PID]->substream_id);
                  strcat(level3, tmpStr);
                }
            }
            else
               strcat(level3, "o");
        }
      }
    }
  } while (
            // (i != MPEG_PROGRAM_END_CODE) && 
            (!end_bs()));

  if ((verbose_level > 2) && (strlen(level3)))
    zprintf(3, level3, 0);

  printf("\n\nSummary:");
  printf("\n\nPkt Errors= %d ReSyncs\n", uResync_Ctr);
  printf("\nMPEG Transport Packets= %u\n\n", transport_packets);
  
  for (i = 0; i < 8192; i++)
  {
    if (streams[i])
    {
      if (streams[i]->stream_packets)
      {
        switch (streams[i]->stream_id)
        {
          case PROGRAM_STREAM_MAP:
            printf("PID %04X, Program Stream Map packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case PRIVATE_STREAM_1:
            printf("PID %04X, Private Stream 1 packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            //if (vob_flag)
            {
              if (streams[i]->substream_packets)
              {
                iPayload = streams[i]->substream_id;
                if ((iPayload >= SUBSTREAM_AC3_0) && (iPayload <= SUBSTREAM_AC3_8))
                    printf("    AC3 Audio stream %u packets= %u, total bytes= %u\n", iPayload - SUBSTREAM_AC3_0, streams[i]->substream_packets, streams[i]->ac3_bytes);
                else
                if ((iPayload >= SUBSTREAM_DTS_0) && (iPayload <= SUBSTREAM_DTS_8))
                    printf("    DTS Audio stream %u packets= %u, total bytes= %u\n", iPayload - SUBSTREAM_DTS_0, streams[i]->substream_packets, streams[i]->dts_bytes);
                else
                if ((iPayload >= SUBSTREAM_PCM_0) && (iPayload <= SUBSTREAM_PCM_F))
                    printf("    PCM Audio stream %u packets= %u, total bytes= %u\n", iPayload - SUBSTREAM_PCM_0, streams[i]->substream_packets, streams[i]->pcm_bytes);
                else
                if ((iPayload >= SUBSTREAM_SUBPIC_0) && (iPayload <= SUBSTREAM_SUBPIC_1F))
                    printf("    Subpicture stream %u packets= %u, total bytes= %u\n", iPayload - SUBSTREAM_SUBPIC_0, streams[i]->substream_packets, streams[i]->subpic_bytes);
                else
                    printf("    Substream 0x%02X packets= %u, total bytes= %u\n", iPayload, streams[i]->substream_packets, streams[i]->substream_bytes);
              }
            }
            break;
          case PRIVATE_STREAM_2:
            printf("PID %04X, Private Stream 2 packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case ECM_STREAM:
            printf("PID %04X, ECM Stream packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case EMM_STREAM:
            printf("PID %04X, EMM Stream packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case PROGRAM_STREAM_DIRECTORY:
            printf("PID %04X, Program Stream Directory packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case DSMCC_STREAM:
            printf("PID %04X, DSMCC Stream packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case ITUTRECH222TYPEE_STREAM:
            printf("PID %04X, ITU-T Rec. H.222.1 type E packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          case PADDING_STREAM:
            printf("PID %04X, Padding Stream packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
            break;
          default:
            iPayload = streams[i]->stream_id;
            if ((iPayload >= 0xE0) && (iPayload <= 0xEF))
              printf("PID %04X, Video stream %d packets= %u, total bytes= %u\n", i, iPayload - 0xE0, streams[i]->stream_packets, streams[i]->stream_bytes);
            else
              if ((iPayload >= 0xC0) && (iPayload <= 0xDF))
                printf("PID %04X, MPEG Audio stream %d packets= %u, total bytes= %u\n", i, iPayload - 0xC0, streams[i]->stream_packets, streams[i]->stream_bytes);
              else
                if (!iPayload)
                {
                  switch (i)
                  {
                    case PROGRAM_ASSOCIATION_TABLE:
                      printf("PID %04X, Program Association Table packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
                      break;
                    case CONDITIONAL_ACCESS_TABLE:
                      printf("PID %04X, Conditional Access Table packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
                      break;
                    case NULL_PACKET:
                      printf("PID %04X, Null packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
                      break;
                    default:
                      printf("PID %04X, Other packets= %u, total bytes= %u\n", i, streams[i]->stream_packets, streams[i]->stream_bytes);
                      break;
                  }
                }
                else
                  printf("PID %04X, Other stream 0x%02X packets= %u, total bytes= %u\n", i, iPayload, streams[i]->stream_packets, streams[i]->stream_bytes);
        }
      }
      free(streams[i]);
    }
  }
  finish_getbits();
}





//------------------------------------------------------



void zprintf(int level, char *tmpStr, int value)
{
  if (verbose_level > 2) // Packet type reports wanted
  {
    if (level > 2) // Packet type report line
    {
      if ((col + strlen(tmpStr) > 76)
      || (tmpStr[2] != oldLevel3[2]))
      {
        col = 0;
        printf("\n");
      }
      printf(tmpStr);
      col += strlen(tmpStr);
      strcpy(oldLevel3, tmpStr);
    }
  }
  else
    if (level <= verbose_level)
      printf(tmpStr, value);

  //  fflush(stdout);
}





//-----------------------------

void PKT_TYPE_Reports()
{
              if ((stream_id >= 0xE0) && (stream_id <= 0xEF))
              {
                strcat(level3, "v");
                sprintf(tmpStr, "%d", stream_id - 0xE0);
                strcat(level3, tmpStr);
              }
              else
              if ((stream_id >= 0xC0) && (stream_id <= 0xDF))
              {
                  strcat(level3, "a");
                  sprintf(tmpStr, "%d", stream_id - 0xC0);
                  strcat(level3, tmpStr);
              }
              else
              if (stream_id == PRIVATE_STREAM_1)
              {
                  strcat(level3, "1");
                  /*
                  if (vob_flag)
                  {
                      sprintf(tmpStr, "%02X", substream_id);
                      strcat(level3, tmpStr);
                  }
                  */
              }
              else
              if (stream_id == PRIVATE_STREAM_2)
              {
                  strcat(level3, "2");
              }
              else
                    strcat(level3, "o");
}

//-----------------------------


void Field_level_PktHdr()
{
  if (verbose_level || iFirstPktOfSubStream)
  {
     printf("\n  Stream_id=x%02X  ", stream_id);

     switch (stream_id)
     {
            case PROGRAM_STREAM_MAP:
              printf("Program Stream Map, ");
              break;
            case PRIVATE_STREAM_1:
                printf("Private Stream 1,");
              break;
            case PRIVATE_STREAM_2:
              printf("Private Stream 2, ");
              break;
            case ECM_STREAM:
              printf("ECM Stream, ");
              break;
            case EMM_STREAM:
              printf("EMM Stream, ");
              break;
            case PROGRAM_STREAM_DIRECTORY:
              printf("Program Stream Directory, ");
              break;
            case DSMCC_STREAM:
              printf("DSMCC Stream, ");
              break;
            case ITUTRECH222TYPEE_STREAM:
              printf("ITU-T Rec. H.222.1 type E, ");
              break;
            case PADDING_STREAM:
                printf("Padding Stream,   ");
               break;
            default:
              if ((stream_id >= 0xE0) && (stream_id <= 0xEF))
              {
                printf("*VIDEO* %d, ", stream_id - 0xE0);

              }
              else
                if ((stream_id >= 0xC0) && (stream_id <= 0xDF))
                {
                  printf("MPEG *AUDIO* %d ", stream_id - 0xC0);

                }
                else
                     printf("miscellaneous ");

           }

           printf("   Pkt#%02d @ %02d", (stream_packets[stream_id] - 1),
                                    pack_packets);
           if (iFirstPktOfSubStream)
               printf("\n\n");
  }

}



//-----------------------------

void SYSHDR_Anl()
{
  unsigned long uTmp1, uTmp2; // , uTmp3;

  system_headers++;

  uFromB =  (unsigned int)(bitcount() / 8 - 4);


        zprintf(1, "\nSYStem_header_start_code= %08X", uStartCode);
        zprintf(2, "   header_length= %d,", getbits(16));
        zprintf(2, "   m%d\n", get1bit());

        zprintf(2, "  rate_bound= %d,", getbits(22));
        zprintf(2, "  m%d", get1bit());
        zprintf(2, "  audio_bound= %d,",getbits(6));
        zprintf(2, "  fixed_flag= %d,",get1bit());
        zprintf(2, "  CSPS_flag= %d\n", get1bit());
        zprintf(2, "  system_audio_lock_flag= %d", get1bit());
        zprintf(2, "  system_video_lock_flag= %d,",get1bit());
        zprintf(2, "  m%d", get1bit());
        zprintf(2, "  video_bound= %d\n", getbits(5));

        if (mpeg2)
        {
           zprintf(2, "  packet_rate_restriction_flag= %d,", get1bit());
           zprintf(2, "  reserved_bits= %d\n", getbits(7));
        }
        else
           zprintf(2, "  reserved_bits= %d\n", getbits(8));

        while (look_ahead(1) == 1)
        {
          zprintf(2, "  Stream_id= %02X ", getbits(8));
          zprintf(2, " 11=%02d", getbits(2));
          zprintf(2, "  P-STD_buffer_bound_scale=%d", get1bit());
          zprintf(2, " P-STD_buffer_size_bound=%d\n", getbits(13));
        }
        if (verbose_level > 2)
        {
          printf("\n");
          strcat(level3, "S");
        }


  uTmp1 =  (unsigned int)(bitcount() / 8 - 4);
  uTmp2 = uTmp1 - uFromB;
  stream_bytes[stream_id] += uTmp2;



}


//-----------------------------

unsigned int uSCRa, uSCRb, uSCRc, uSCRx;
unsigned int uMark1, uMark2, uMark3;

void Pack_SCR_FLOAT();
void Pack_SCR_Info();
void Pack_SCR_Parts();


void PACK_Anl()
{
  int iLen;
  unsigned long uTmp1, uTmp2; // , uTmp3;


  pack_packets++;

  uFromB =  (unsigned int)(bitcount() / 8 - 4);

  if (PrevPack_bit_count == 0.0)
     uPackSize = 0;
  else
     uPackSize = ((unsigned long)(bitcount() - PrevPack_bit_count)) >> 3;

  PrevPack_bit_count = bitcount();

  if (verbose_level < 3)
       zprintf(1, "\nPACK #%d ", pack_packets - 1);

  zprintf(1, "StartCd=%04X ", uStartCode);

  Pack_Version_Bits = getbits(2);

  if (!Pack_Version_Bits  && mpeg2 && !vob_flag)
  {
          zprintf(1,"\nPACK has different Mpeg Version: %d\n",
                      (Pack_Version_Bits+1));
          //beep(200,200);
          fflush(stdout);
  }

  if (Pack_Version_Bits)
      mpeg2 = true;
  else
      mpeg2 = false;


  if (mpeg2)
      zprintf(2, " 01=%d", Pack_Version_Bits);
  else
  {
      Pack_Version_Bits = (Pack_Version_Bits << 2) | getbits(2);
      zprintf(2, " 0010=%d", Pack_Version_Bits);
  }

  // Pack size stats
  if (uPackSize  != 0)
  {
            int cTmp1;
            cTmp1 = ' ';
            if (uPackSize == 2048)
                uDVDfit++;
            else
            if (uPackSize == 2324)
                uVCDfit++;
            else
            {
               cTmp1 = '*';
               if (uPackSize == 14)
                   uOrphans++;
               else
                   uOtherPacks++;
            }
            zprintf(1, " %c", cTmp1);               // RJ was level 2
            zprintf(1, "PrevSz=%d ", uPackSize ); // RJ was level 2
  }

  pos_prt(1, verbose_level);

  uPackPics = uPIC_ctr - uPackPrevPics;
  uPackPrevPics = uPIC_ctr;

  zprintf(1, "Pics=%u ", uPackPics ); 

  // SCR

  Pack_SCR_Parts();

  if (verbose_level == 1) // Basic format uses floating point
  {
        Pack_SCR_FLOAT();
  }
  else
  {
        Pack_SCR_Info();
  }

  // Other fields

  zprintf(2, "  m%d\n", get1bit());
  zprintf(2, "     program_mux_rate= %d,", getbits(22) );
  zprintf(2, " m%d ", get1bit());

  if (mpeg2)
  {
      zprintf(2, "%d,", get1bit());
      zprintf(2, "  res=%d ", getbits(5));
      iLen = getbits(3);
      zprintf(2, "  pack_stuffing_length= %d\n", iLen);

      if (iLen > 0)
      {
        zprintf(2, "    Stuffing=x%02X", getbits(8));
        for (iPayload = 1;
             iPayload < iLen;
             iPayload++)
        {
               zprintf(2, " x%02X", getbits(8));
        }
      }
  }

  if (verbose_level > 2)
  {
      if (strlen(level3))
          zprintf(3, level3, 0);
      strcpy(level3, " P");
  }
  else
  if (verbose_level > 0)
      printf("\n");


  uTmp1 =  (unsigned int)(bitcount() / 8 - 4);
  uTmp2 = uTmp1 - uFromB;
  stream_bytes[stream_id] += uTmp2;


}


//------------------

void Pack_SCR_Parts()
{

          uSCRa = getbits(3);
          uMark1 = get1bit();

          uSCRb = getbits(15);
          uMark2 = get1bit();

          uSCRc = getbits(15);
          uMark3 = get1bit();

          if (mpeg2)
          {
              uSCRx = getbits(9);
              if (uSCRx)
                  uScrExtn_Ctr++;
          }
          else
              uSCRx = 0;

          //uSCR_Major = uSCRa;
          //uSCR_Major = uSCR_Major << 30;
          //uSCR_Major |= (uSCRb << 15);
          //uSCR_Major |= uSCRc;

          uSCR_Major = (uSCRa<<30) | (uSCRb<<15) | uSCRc ;

          if ( ! uSCR_Prev)
             iDiff_TC = 0;
          else
             iDiff_TC = uSCR_Major - uSCR_Prev;
}



//----------------------


void Pack_SCR_FLOAT()
{

  // Convert SCR fields into a floating point value

  if (uSCRa & 0x00000004)        // What is this ?
      dScr_value = 4294967296.0; // What is this ?
  else
      dScr_value = 0.0;

  dScr_value += (long double) uSCR_Major;
  dScr_value *= 300.0;

  if (mpeg2)
  {
      dScr_value += (long double)(uSCRx);
  }


  Pack_SCR_Info();

  dScr_value /= 300.0;


}




void Pack_SCR_Info()
{
 unsigned char cTmp1;
 unsigned int uExpected;

  if (verbose_level != 3  || iDiff_TC < 0 || iDiff_TC > 45000 )
  {

    if (strlen(level3))
    {
        zprintf(3, level3, 0);
        strcpy(level3, "");
    }

    uExpected =  uPackPics*1001*90000 / uFRate1001; // Allow for very long packs
    uExpected += 45000; // Allow for extra half a second difference

    if (iDiff_TC > uExpected || iDiff_TC < -45000)
    {
        cTmp1 = '*';
        uGAP_ctr++;
        printf("\nGAP#%d   %d ms = %d tk", uGAP_ctr,
                            (iDiff_TC / 90), iDiff_TC);
         if (iDiff_TC < 0)
             printf("\n\n*** SEQ ERR***\n\n");
    }
    else
        cTmp1 = ' ';

    if (verbose_level || cTmp1 > ' ')
        printf("\n");

    if (verbose_level >= 3)
        zprintf(1,"#%04d  ",  pack_packets);

    if (verbose_level || cTmp1 > ' ')
    {
       if (verbose_level == 1)
           printf("SCR= %08Lf ms  %09.3Lf tk  Inc=%07u tk%c x=%03d",
                dScr_value / 27000.0, dScr_value, iDiff_TC, cTmp1, uSCRx);
       else
       {
         printf("SCR= %07d ms   %09u tk  Inc=%06d tk%c x=%03d",
                    uSCR_Major / 90, uSCR_Major, iDiff_TC, cTmp1, uSCRx);

         if (verbose_level == 0)
              printf("\n");
         else
         {
           //zprintf(2, "  SCR_base=%1d", uSCRa);
           zprintf(2, " [%1d", uSCRa);
           //zprintf(2, "  system_clock_reference_base = %1d", iTmp1);
           zprintf(2, ",%3d", uSCRb);
           zprintf(2, ",%05d", uSCRc);
           zprintf(2, " %3d]", uSCRx);
           zprintf(2, " m%1d", uMark1);
           zprintf(2, "%1d", uMark2);
           zprintf(2, "%1d", uMark3);
         }
       }
    }
  }

  if (uStartTC[256] == 0xFFFFFFFF) 
      uStartTC[256] =  uSCR_Major;

  uSCR_Prev = uSCR_Major;


} 





//--------------------------------------

 void GOP_Stats_Inc()
 {
   unsigned int uTmp1;

   // HOW MANY FRAMES IN Previous GOP

   if (uRel_ctr == 13) uGOPspanDVDPAL++;  else
   if (uRel_ctr == 17) uGOPspanDVDNTSC++; else
   if (uRel_ctr ==  1) uGOPspan1++;       else
   if (uRel_ctr ==  0) uGOPspan0++;       else
   if (uRel_ctr  > 99) uGOPspanKDVD++;    else
   if (uRel_ctr  > 18) uGOPspanLONG++;    else
   if (uRel_ctr  < 12) uGOPspanSHORT++;   else
                       uGOPspanMISC++;

   if (uRel_ctr  < 601)
       uTmp1 = uRel_ctr;
   else
       uTmp1 = 601;

   if (uGOP_ctr)
       uGOP_Frames[uTmp1]++;

   if (uPTS_Major > uGOP_PrevPTS)
   {
      uGOP_Diff_PTS_ms = ((uPTS_Major - uGOP_PrevPTS) + 45 ) / 90;
   }
   else
      uGOP_Diff_PTS_ms = 0;

   uGOP_PrevPTS = uPTS_Major;


}



//----------------------

int iHexPrt_Ctr;

void Hex_Prt(int P_Len)
{
  int iLen, iTmp1;

  iLen = P_Len;

  for (iHexPrt_Ctr = 0; 
       iHexPrt_Ctr < iLen && iPayload < iRemain;
       )
  {
      iTmp1 = getbits(8);
      if ((iHexPrt_Ctr & 0x0F) == 0)
      {
          printf("\n     Data = ");
      }

      printf("%02X ", iTmp1);
      if ((iHexPrt_Ctr & 0x03) == 3)
          printf("- ");
          
      iHexPrt_Ctr++;
      iPayload++;
  }
  printf("\n");


}


//----------------------

char *szAC3Mode[8] =
{
  "1+1", "1/0", "2/0", "3/0", "2/1", "3/1", "2/2", "3/2"
}
;

int iAC3Rate[32]  =
{
  32, 40, 48, 56, 64, 80, 96, 112, 128, 160,
  192, 224, 256, 320, 384, 448, 512, 576, 640,
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
}
;


unsigned int uRate, uMode, u1stFramePtr, uFramesExpected;
int DD_Syncword_Status=0, DD_Quadword_Status=0;
int DD_Frame_ctr;

void DD_Audio_Byte()
{
  int iTmp1;
  char szAligned[16], szRoot[16];

  iTmp1 = getbits(8);

  //iPayload++;
  iHexPrt_Ctr++;

  if (DD_Syncword_Status  == 2
  ||  DD_Syncword_Status  == 3
  ||  DD_Syncword_Status  == 5)
      DD_Syncword_Status++;
  else
  if (DD_Syncword_Status == 4)
  {
       uRate     = (iTmp1>>1) & 0x1f;
       DD_Syncword_Status   = 5;
  }
  else
  if (DD_Syncword_Status == 6)
  {
       uMode     = (iTmp1>>5) & 0x07;

       DD_Syncword_Status  = 0;

       if (!DD_Frame_ctr)
       {
           if (iPayload == 10)
               strcpy(szAligned, "Aligned");
           else
               strcpy(szAligned, "UNALIGNED");

           if ((iPayload-9) ==  u1stFramePtr)
               strcpy(szRoot, "Root OK");
           else
               strcpy(szRoot, "BAD ROOT");
       }
       else
       {
         szAligned[0] = 0;  szRoot[0] = 0;
       }

       printf("\n     * %04d  %02ukbps  %s   %s  %s     ",
                 (iPayload-10), iAC3Rate[uRate], szAC3Mode[uMode],
                                                szAligned,
                                                szRoot);

       // iHexPrt_Ctr = 16; // print a little more

       DD_Frame_ctr++;

  }
  else
  if (iTmp1 == 0x0B)
      DD_Syncword_Status  = 1;
  else
  if (DD_Syncword_Status == 1
  &&  iTmp1  == 0x77)
  {
      DD_Syncword_Status  = 2;
  }
  else
  {
      DD_Syncword_Status  = 0;
  }


  if (verbose_level > 0
  &&  iHexPrt_Ctr <= 20 && ! DD_Frame_ctr)
  {
      if (iPayload == 4)
          printf("     Data = ");

      printf("%02X ", iTmp1);
      if (iHexPrt_Ctr && 7 == 4)
          printf("- ");
  }

}


//----------------------


void DD_Audio_Info(char Codec_Abbr[4], int iTrackRel)
{
  int iRemain;
  //int iTmp1, iTmp2, iTmp3, iTmp4;

  DD_Frame_ctr=0;  iHexPrt_Ctr=0;
  // iPayload=1;
  iRemain = PES_packet_length - bytes_used; // BYTE-ME - 3;

  if (verbose_level > 0 || substream_packets[substream_id] < 1)
  {
      printf("  Substream= 0x%02X   ", substream_id);
      printf("%s Audio Trk#%d\n",      Codec_Abbr, iTrackRel);
      uFramesExpected =  getbits(8);
      u1stFramePtr = getbits(16);
      iPayload += 3;
      printf("     #SyncWords= %02u, ", uFramesExpected);
      printf("1st Offset= %04u =0x%04X\n", u1stFramePtr, u1stFramePtr);
  }


  iHexPrt_Ctr = 0;

  /*
  if (verbose_level  && iRemain > 4 )
  {
      //printf("    Data = ");

      while (iPayload < iRemain && iPayload < 20)
      {
              iTmp1 = getbits(8);
              iTmp2 = getbits(8);
              iTmp3 = getbits(8);
              iTmp4 = getbits(8);

              iPayload+=4;

              printf("%02X %02X %02X %02X - ",
                                       iTmp1, iTmp2, iTmp3, iTmp4);
      }

      printf("\n");
  }
  */


  for (; iPayload < iRemain; iPayload++)
  {
    if (verbose_level  || substream_packets[substream_id] < 1)
       DD_Audio_Byte();
    else
        getbits(8);
  }

  if (verbose_level  || substream_packets[substream_id] < 1)
      printf("\n");

}



//----------------------


void PCM_Audio_Info()    // LPCM
{
  int iRemain;
  int iTmp1, iTmp2, iTmp3, iTmp4;
  int iLPCM_ctl, iFlags; // , iKbRate;
  int iEmphasis, iMute, iReserved;
  unsigned int uQWord_Len, uDynamicRange;
  unsigned int uSamplingRate, uBitsPerSample, uChannel_ix, uRelFrame;
  char *szOnOff[2] = {"Off", "On"};

  DD_Frame_ctr=0;  iHexPrt_Ctr=0;
  //iPayload=1;

  iRemain = PES_packet_length - bytes_used; // BYTE-ME - 3;

  uFramesExpected = getbits(8);  // No of Frames starting in this packet (does not include tail from previous packet, but does include last partial frame in this packet)
  u1stFramePtr    = getbits(16); // Offset to start of 1st frame for this PTS = first access unit pointer

  iFlags = getbits(8); // Flags:
                      //    audio emphasis on-off        1 bit
                      //    audio mute on-off            1 bit
                      //    reserved                     1 bit
                      //    audio frame number           5 bits (within Group of Audio frames).
  iEmphasis = iFlags & 0x80;
  iMute     = iFlags & 0x40;
  iReserved = iFlags & 0x20;
  uRelFrame = iFlags & 0x1F;

  iLPCM_ctl = getbits(8); // LPCM Attributes:
                   // quantization word length code   2 bits  0 = 16, 1 = 20, 2 = 24, 3 = reserved
                   // audio sampling frequency        2 bits  (48khz = 0, 96khz = 1)
                   // reserved                        1 bit
                   // number of audio channels - 1    3 bit  (e.g. stereo = 1)

  uDynamicRange = getbits(8); // x'80' = off
                          //      Dynamic Range X     3 bits
                          //      Dynamic Range Y     5 bits
                          // linear gain = 2^(4-(X+(Y/30)))
                          //  in dB gain = 24.082 - 6.0206 X - 0.2007 Y


  uQWord_Len = ((iLPCM_ctl >>6) + 4);
  uBitsPerSample = uQWord_Len * 4;

  if ((iLPCM_ctl & 0x30) == 0x10)
  {
          uSamplingRate = 96000;
  }
  else
  {
          uSamplingRate = 48000;
  }


  printf("    Sample Size= %02u, ", uBitsPerSample);

  printf("   Rate= %05u, ", uSamplingRate);
  // iKbRate = uSamplingRate / 1000;

  uChannel_ix      = (iLPCM_ctl & 0x03) + 1;
  if (uChannel_ix == 3)
      uChannel_ix  = 4;
  printf("   Channels= %u\n", uChannel_ix);


  printf("    #frames= %02u, ", uFramesExpected);
  printf("1st offset= %04u =0x%04X\n", u1stFramePtr, u1stFramePtr);

  printf("    Emphasis=%s  Mute=%s  Res1=%d  RelativeFrame=%02u DRC=x%02X  Payload=%d",
                 szOnOff[iEmphasis], szOnOff[iMute], iReserved, uRelFrame,
                                                             uDynamicRange,
                                                               (iRemain-7));


  iPayload += 6;

  iHexPrt_Ctr = 42;

  if (verbose_level)
  {
      while (iPayload < iRemain && iPayload < 36)
      {
        if (iHexPrt_Ctr < 12)
            iHexPrt_Ctr+=4;
        else
        {
           iHexPrt_Ctr = 0;
           if (iPayload < 8)
               printf("\n    Data = ");
           else
               printf("\n           ");
        }

        iTmp1 = getbits(8);
        iTmp2 = getbits(8);
        iTmp3 = getbits(8);
        iTmp4 = getbits(8);
        iPayload+=4;

        printf("%02X%02X %02X%02X - ",
                iTmp1, iTmp2, iTmp3, iTmp4);
      }
  }

  printf("\n");

  for ( ; iPayload < iRemain; iPayload++)
  {
        getbits(8);
  }

  printf("\n");

}


//---------------------------------

void PS1_Info()
{
  int iRemain2;
  int iTrk;
  int iTmp1, iTmp2, iTmp3, iTmp4;
  
  substream_id = getbits(8);
  iPayload=1;

  iFirstPktOfSubStream = !stream_packets[stream_id] && !verbose_level;

  if (uStartTC[substream_id] == 0xFFFFFFFF)
  {
      uStartTC[substream_id] = uStartTC[stream_id];
  }
                    
  uPTS_Prev[substream_id] = uPTS_Prev[stream_id];

  if ((substream_id >= SUBSTREAM_AC3_0) 
  &&  (substream_id <= SUBSTREAM_AC3_8))
  {
       iTrk = substream_id - SUBSTREAM_AC3_0;
       DD_Audio_Info("AC3", iTrk);
  }
  else
  if ((substream_id >= SUBSTREAM_DTS_0) 
   && (substream_id <= SUBSTREAM_DTS_8))
  {
       iTrk = substream_id - SUBSTREAM_DTS_0;
       DD_Audio_Info("DTS", iTrk);
  }
  else
  if ((substream_id >= SUBSTREAM_DDP_0) 
   && (substream_id <= SUBSTREAM_DDP_F))
  {
      iTrk = substream_id - SUBSTREAM_DDP_0;
      DD_Audio_Info("DD+", iTrk);
  }
  else
  if ((substream_id >= SUBSTREAM_PCM_0) 
   && (substream_id <= SUBSTREAM_PCM_F))
  {
     iPayload=1;

     if (verbose_level > 0 || substream_packets[substream_id] < 1)
     {
         printf("  Substream= 0x%02X  ", substream_id);
         printf("LPCM Audio Trk#%d\n",  (substream_id - SUBSTREAM_PCM_0));
         PCM_Audio_Info();
     }
     else
     {
         //for (iPayload = 1; iPayload < PES_packet_length - bytes_used; // BYTE-ME - 3; iPayload++)
         //    getbits(8);

         iRemain2 = PES_packet_length - bytes_used; // BYTE-ME - 3;

         if (iRemain2 > 4 && verbose_level)
         {
            if (verbose_level)
                printf("    Data = ");

            while (iPayload < iRemain2 && iPayload < 20)
            {
                iTmp1 = getbits(8);
                iTmp2 = getbits(8);
                iTmp3 = getbits(8);
                iTmp4 = getbits(8);

                iPayload+=4;

                if (verbose_level)
                    printf("%02X %02X %02X %02X - ",
                                             iTmp1, iTmp2, iTmp3, iTmp4);
            }
                
            if (verbose_level)
                   printf("\n");
         }
                          
         for (; iPayload < iRemain2; 
                iPayload++)
         {
              getbits(8);
         }
     } // end-if Hex group of 8
  } // end-if PCM

  else
  if ((substream_id >= SUBSTREAM_SUBPIC_0) 
   && (substream_id <= SUBSTREAM_SUBPIC_1F))
  {
      if (verbose_level > 0 || substream_packets[substream_id] < 1)
      {
          zprintf(0, "  Substream= 0x%02X  ", substream_id);
          zprintf(0, "SUBTitle Trk#%d\n", substream_id - SUBSTREAM_SUBPIC_0);
      }

      for (iPayload = 1; 
           iPayload < PES_packet_length - bytes_used; // BYTE-ME - 3; 
           iPayload++)
           {
                 getbits(8);
           }
  } // end-if Subtitle

  else
  {
      if (verbose_level > 0 || substream_packets[substream_id] < 1)
      {
            zprintf(0, "  UNK Substream= 0x%02X\n", substream_id);
      }
        
      for (iPayload = 1; 
           iPayload < PES_packet_length - bytes_used; // BYTE-ME - 3; 
           iPayload++)
           {
              getbits(8);
           }

  } // end-else NOT Known
  
                  
}



//---------------------------

void PTS_Info()
{
  long double   dTmp2;
  char cTmp1;
  int iSubField[8], iTmp1;
  unsigned long uTmp2;


                if (verbose_level == 1)
                {
                  getbits(4);
                  uPTS_Major = getbits(3);
                  if (uPTS_Major & 0x00000004)
                    dPts_value = 4294967296.0;
                  else
                    dPts_value = 0.0;
                  uPTS_Major = uPTS_Major << 30;
                  get1bit();
                  uPTS_Major |= (getbits(15) << 15);
                  get1bit();
                  uPTS_Major |= getbits(15);
                  dPts_value += (long double) uPTS_Major;
                  get1bit();

                  if (uPTS_Prev[substream_id] == 0xFFFFFFFF)
                  {
                     iDiff_TC = 0;
                     uStartTC[substream_id] = uPTS_Major;
                  }
                  else
                     iDiff_TC = uPTS_Major - uPTS_Prev[substream_id];

                  uPTS_Prev[substream_id] = uPTS_Major;

                  dTmp2 = dPts_value - dScr_value;

                  if (iDiff_TC > 90000 || iDiff_TC < -45000)
                     cTmp1 = '*';
                  else
                     cTmp1 = ' ';


                  if (verbose_level != 3)
                  {
                    iTmp1 = iDiff_TC / 90;
                    printf("  PTS= %08Lf ms, Inc:%04dT=%03dms %c",
                              (dPts_value / 90.0),
                                iDiff_TC, iTmp1,
                                cTmp1);

                    if (dTmp2 <= 0)
                        printf(", underflow %Lf ms\n", dTmp2);
                    else
                        printf("\n");
                  }
                }
                else
                {
                  //if (verbose_level == 2)
                  //  printf("  PTS fields:");

                  if (strlen(level3))
                  {
                          zprintf(3, level3, 0);
                          strcpy(level3, "");
                  }

                  iSubField[0] = getbits(4);
                  iSubField[1] = getbits(3);
                  iSubField[2] = get1bit();
                  iSubField[3] = getbits(15);
                  iSubField[4] = get1bit();
                  iSubField[5] = getbits(15);
                  iSubField[6] = get1bit();

                  uPTS_Major = (iSubField[1] << 30) | (iSubField[3]<<15)
                              | iSubField[5];

                  if (uPTS_Prev[stream_id] == 0xFFFFFFFF)
                  {
                     iDiff_TC = 0;
                     uStartTC[stream_id] = uPTS_Major;
                  }
                  else
                     iDiff_TC = uPTS_Major - uPTS_Prev[stream_id];

                  uPTS_Prev[stream_id] = uPTS_Major;

                  if (iDiff_TC > 90000 || iDiff_TC < -45000)
                     cTmp1 = '*';
                  else
                     cTmp1 = ' ';

                  uTmp2 = uPTS_Major / 90;

                  if (verbose_level != 3)
                  {
                     if (strlen(level3))
                     {
                        zprintf(3, level3, 0);
                        strcpy(level3, "");
                     }

                     if (verbose_level > 3)
                         strcat(level3, "PTS");
                     if (verbose_level || cTmp1 > ' ')
                         printf("\n  PTS= %08dms  =%07uT  Inc:%03dms =%04dT %c f=%03d",
                                        uTmp2, uPTS_Major,
                                        (iDiff_TC/90), iDiff_TC,
                                        cTmp1,
                                   ( (int)(uTmp2) - (int)(uSCR_Prev/90) ) );
                     zprintf(2, "  x(%1X", iSubField[1]);
                     zprintf(2, ",%2X", iSubField[3]);
                     zprintf(2, ",%2X)", iSubField[5]);
                     zprintf(2, " %02X", iSubField[0]);
                     zprintf(2, " m%X", iSubField[2]);
                     zprintf(2, "%X", iSubField[4]);
                     zprintf(2, "%X ", iSubField[6]);
                     if (verbose_level || cTmp1 > ' ')
                         printf("\n");
                  }

                }
                bytes_used += 5;

}

/*
                if (verbose_level == 1)
                {
                  getbits(4);
                  uPTS_Major = getbits(3);
                  if (uPTS_Major & 0x00000004)
                    dPts_value = 4294967296.0;
                  else
                    dPts_value = 0.0;
                  uPTS_Major = uPTS_Major << 30;
                  get1bit();
                  uPTS_Major |= (getbits(15) << 15);
                  get1bit();
                  uPTS_Major |= getbits(15);
                  dPts_value += (long double) uPTS_Major;
                  get1bit();
                  if (dPts_value <= dScr_value)
                    printf("\n  PTS= %08Lf ms underflow, ", dPts_value / 90.0);
                  else
                    printf("\n  PTS= %08Lf ms, ", dPts_value / 90.0);
                  getbits(4);
                  uDTS_Major = getbits(3);
                  if (uDTS_Major & 0x00000004)
                    dDts_value = 4294967296.0;
                  else
                    dDts_value = 0.0;
                  uDTS_Major = uDTS_Major << 30;
                  get1bit();
                  uDTS_Major |= (getbits(15) << 15);
                  get1bit();
                  uDTS_Major |= getbits(15);
                  dDts_value += (long double) uDTS_Major;
                  get1bit();
                  if (dDts_value <= dScr_value)
                    printf("\n  DTS= %08Lf ms underflow\n", dDts_value / 90.0);
                  else
                    printf("\n  DTS= %08Lf ms\n", dDts_value / 90.0);
                }
                else
                {
                  //if (verbose_level == 2)
                  //  printf("  PTS fields:");
                  iSubField[0] = getbits(4);
                  iSubField[1] = getbits(3);
                  iSubField[2] = get1bit();
                  iSubField[3] = getbits(15);
                  iSubField[4] = get1bit();
                  iSubField[5] = getbits(15);
                  iSubField[6] = get1bit();

                  uPTS_Major = (iSubField[1] << 30) | (iSubField[3]<<15)
                              | iSubField[5];

                  if (uPTS_Prev[stream_id] != 0xFFFFFFFF)
                  {
                     iDiff_TC = 0;
                     uStartTC[stream_id] = uPTS_Major;
                  }
                  else
                     iDiff_TC = uPTS_Major - uPTS_Prev[stream_id];

                  uPTS_Prev[stream_id] = uPTS_Major;

                  if (iDiff_TC > 90000 || iDiff_TC < -45000)
                     cTmp1 = '*';
                  else
                     cTmp1 = ' ';

                  uTmp2 = uPTS_Major / 90;


                  if (verbose_level != 3)
                  {
                       if (strlen(level3))
                       {
                          zprintf(3, level3, 0);
                          strcpy(level3, "");
                       }

                       if (verbose_level || cTmp1 > ' ')
                           printf("\n  PTS= %08d ms  %07u tk  Inc:%07d tk%c f=%03d",
                                         uTmp2, uPTS_Major, iDiff_TC, cTmp1,
                                   ( (int)(uTmp2) - (int)(uSCR_Prev/90) ) );
                      zprintf(2, " x(%1X", iSubField[1]);
                      zprintf(2, ",%5X", iSubField[3]);
                      zprintf(2, ",%5X)", iSubField[5]);
                      zprintf(2, " 0010= %X", iSubField[0]);
                      zprintf(2, " m%X", iSubField[2]);
                      zprintf(2, "%X", iSubField[4]);
                      zprintf(2, "%X ", iSubField[6]);
                      if (verbose_level || cTmp1 > ' ')
                          printf("\n");
                  }
*/


void DTS_Info()
{
  //long double   dTmp2;
  char cTmp1;
  int iSubField[8];
  // unsigned long uTmp0, uTmp1;

                  //if (verbose_level == 2)
                  //  printf("  DTS fields:");

                  iSubField[0] = getbits(4);
                  iSubField[1] = getbits(3);
                  iSubField[2] = get1bit();
                  iSubField[3] = getbits(15);
                  iSubField[4] = get1bit();
                  iSubField[5] = getbits(15);
                  iSubField[6] = get1bit();

                  uDTS_Major = (iSubField[1] << 30) | (iSubField[3]<<15)
                              | iSubField[5];
                  if ( ! uDTS_Prev[stream_id])
                     iDiff_TC = 0;
                  else
                     iDiff_TC = uDTS_Major - uDTS_Prev[stream_id];
                  uDTS_Prev[stream_id] = uDTS_Major;

                  if (iDiff_TC > 90000 || iDiff_TC < -45000)
                     cTmp1 = '*';
                  else
                     cTmp1 = ' ';


                  if (verbose_level != 3)
                  {
                      if (verbose_level || cTmp1 > ' ')
                          printf("  DTS= %08dms  =%07uT  Inc.%03ums =%04dT %c         ",
                                (uDTS_Major / 90), uDTS_Major,
                                  (iDiff_TC / 90), iDiff_TC,
                                                   cTmp1);

                      zprintf(2, " x(%1X", iSubField[1]);
                      zprintf(2, ",%1X", iSubField[3]);
                      zprintf(2, ",%2X)", iSubField[5]);
                      zprintf(2, " %02X", iSubField[0]);
                      zprintf(2, " m%X", iSubField[2]);
                      zprintf(2, "%X", iSubField[4]);
                      zprintf(2, "%X ", iSubField[6]);
                      if (verbose_level || cTmp1 > ' ')
                          printf("\n");
                  }

                  if (verbose_level > 3)
                    strcat(level3, "DTS");

                  bytes_used += 5;
}




// MPEG-1 PTS/DTS   ------------------------------------------

void PTS_Mpeg1_Basic()
{
                  zprintf(2, "    0010= %d", getbits(4));
                  zprintf(2, "    PTS[32..30]= %5d,", getbits(3));
                  zprintf(2, "  m%d", get1bit());
                  zprintf(2, "  [29..15]= %5d,", getbits(15));
                  zprintf(2, "  m%d", get1bit());
                  zprintf(2, "  [14..00]= %5d,", getbits(15));
                  zprintf(2, "  m%d\n", get1bit());
}



void PTS_Mpeg1_Full()
{
                  getbits(4);
                  uPTS_Major = getbits(3);
                  if (uPTS_Major & 0x00000004)
                    dPts_value = 4294967296.0;
                  else
                    dPts_value = 0.0;
                  uPTS_Major = uPTS_Major << 30;
                  get1bit();
                  uPTS_Major |= (getbits(15) << 15);
                  get1bit();
                  uPTS_Major |= getbits(15);
                  dPts_value += (long double) uPTS_Major;
                  get1bit();
                  printf("\n  PTS= %08Lf ms", dPts_value / 90.0);

                  if (dPts_value <= dScr_value)
                    printf(" underflow");


}


void DTS_Mpeg1_Basic()
{
                  zprintf(2, "  0001= %d", getbits(4));
                  zprintf(2, "  DTS[32..30]= %5d,", getbits(3));
                  zprintf(2, "  m%d", get1bit());
                  zprintf(2, "  [29..15]= %5d,", getbits(15));
                  zprintf(2, "  m%d", get1bit());
                  zprintf(2, "  [14..00]= %5d,", getbits(15));
                  zprintf(2, "  m%d\n\n", get1bit());
}


void DTS_Mpeg1_Full()
{

                    getbits(4);
                    uDTS_Major = getbits(3);
                    if (uDTS_Major & 0x00000004)
                      dDts_value = 4294967296.0;
                    else
                      dDts_value = 0.0;
                    uDTS_Major = uDTS_Major << 30;
                    get1bit();
                    uDTS_Major |= (getbits(15) << 15);
                    get1bit();
                    uDTS_Major |= getbits(15);
                    dDts_value += (long double) uDTS_Major;
                    get1bit();

                    printf("\n  DTS= %08Lf ms", dDts_value / 90.0);

                    if (dDts_value <= dScr_value)
                      printf(" underflow");

}

void PES_HDR_Mpeg1()
{
              /* flush stuffing bytes */
              while (look_ahead(1) == 1)
              {
                getbits(8);
                bytes_used++;
              }
              zprintf(2, "  read %d stuffing bytes\n", bytes_used);

              if (look_ahead(2) == 1)
              {
                if (verbose_level == 2)
                  printf("  P-STD buffer fields:");
                zprintf(2, "    01= %d\n", getbits(2));
                zprintf(2, "    P-STD_buffer_scale= %d,", get1bit());
                zprintf(2, "    P-STD_buffer_size= %d\n", getbits(13));
                bytes_used += 2;
                if (verbose_level > 3)
                  strcat(level3, " P-STD");
              }

              if (look_ahead(4) == 2)
              {
                if (verbose_level == 1)
                {
                  PTS_Mpeg1_Full();
                  printf("\n");
                }
                else
                {
                  if (verbose_level > 3)
                    strcat(level3, "PTS");
                  else
                  if (verbose_level == 2)
                    printf("  PTS fields:");

                  if (strlen(level3))
                  {
                          zprintf(3, level3, 0);
                          strcpy(level3, "");
                  }

                  PTS_Mpeg1_Basic();
                }

                bytes_used += 5;
              }
              else
              if (look_ahead(4) == 3)  //  PTS+DTS (Mpeg-1)
              {
                 if (verbose_level == 1)
                 {
                    PTS_Mpeg1_Full();
                    printf(", ");

                    DTS_Mpeg1_Full();
                    printf("\n");

                 } // end-if Verbose_level 1
                 else
                 {
                    if (verbose_level == 2)
                      printf("  PTS fields:");

                    PTS_Mpeg1_Basic();

                    if (verbose_level == 2)
                      printf("  DTS fields:");
                    DTS_Mpeg1_Basic();

                    if (verbose_level > 3)
                      strcat(level3, "PTSDTS");

                 } // end-else NOT Verbose_level 1

                 bytes_used += 10;

              } // end-if  PTS+DTS (Mpeg-1)

              else
              {
                  if (verbose_level == 1)
                    printf("\n");

                  iPayload = getbits(8);
                  if (iPayload != 0x0f)
                  {
                    printf("\n    ****\n    ****  Invalid bits in MPEG-1 file\n    ****\n\n");
                    // exit (1);
                  }
                  zprintf(2, "  no_timestamps marker= %d\n", iPayload);
                  bytes_used++;
              }
}




//----------------------------------------------------------------------

void PES_HDR_Mpeg2()
{
  int iTmp1, iTmp2, iTmp3, iTmp4, iTmp5, iTmp6;

  iTmp1 = getbits(2);
  iTmp2 = getbits(2);
  iTmp3 = get1bit();
  iTmp4 = get1bit();
  iTmp5 = get1bit();
  iTmp6 = get1bit();

  PTS_DTS_flags             = getbits(2);
  ESCR_flag                 = get1bit();
  ES_rate_flag              = get1bit();
  DSM_trick_mode_flag       = get1bit();
  additional_copy_info_flag = get1bit();
  PES_CRC_flag              = get1bit();
  PES_extension_flag        = get1bit();

  PES_header_data_length = getbits(8);

  bytes_used+=3; // BYTE-ME

  zprintf(2, "  10= %d",              iTmp1);

  if (!iTmp2 && !iTmp3 && !iTmp4 && !iTmp5 && !iTmp6)
  {
                  if (verbose_level == 2)
                      printf("  Flags1=000");
  }
  else
  {
                  zprintf(2, "  PES_scrambling= %d,", iTmp2);
                  zprintf(2, "  PES_priority= %d",    iTmp3);
                  zprintf(2, "  data_alignment= %d",  iTmp4);
                  zprintf(2, "  (c)=%d,",             iTmp5);
                  zprintf(2, " copy=%d\n",            iTmp6);
  }

  if (!ESCR_flag && !ES_rate_flag && !DSM_trick_mode_flag
  &&  !additional_copy_info_flag && !PES_CRC_flag
  &&  !PES_extension_flag)
  {
                  if (verbose_level == 2)
                      printf("  Flags2=000");
  }
  else
  {
                 zprintf(2, "  ESCR_f=%d", ESCR_flag);
                 zprintf(2, "  ES_rate_f=%d", ES_rate_flag);
                 zprintf(2, "  DSM_trick_f=%d", DSM_trick_mode_flag);
                 zprintf(2, "  addit_copy_f=%d\n", additional_copy_info_flag);

                 zprintf(2, "  PES_CRC_flag=%d,", PES_CRC_flag);
                 zprintf(2, "  PES_extension_flag= %d\n", PES_extension_flag);
  }

  if (!PTS_DTS_flags && !PES_header_data_length)
  {
                  if (verbose_level == 2)
                      printf("  Flags3=000\n");
  }
  else
  {
                zprintf(2, "\n  Flags: PTS_DTS_f= %d", PTS_DTS_flags);
                zprintf(2, "  PES_HEADER_data_length= %d \n", PES_header_data_length);
  }

}


//-----------------------


void PES_Minor()
{
  int iTmp1;


              if (ESCR_flag == 1)
              {
                if (verbose_level == 2)
                  printf("  ESCR fields:");

                zprintf(2, "  res= %d", getbits(2));
                zprintf(2, "  ESCR_base[32..30]= %5d,", getbits(3));
                zprintf(2, "  m%d", get1bit());
                zprintf(2, "  ESCR_base[29..15]= %5d,", getbits(15));
                zprintf(2, "  m%d", get1bit());
                zprintf(2, "  ESCR_base[14..00]= %5d,", getbits(15));
                zprintf(2, "  m%d", get1bit());
                zprintf(2, "  ESCR_extension= %d,", getbits(9));
                zprintf(2, "  m%d\n", get1bit());
                bytes_used += 6;
              }

              if (ES_rate_flag == 1)
              {
                if (verbose_level == 2)
                  printf("  ES rate fields:");
                zprintf(2, "    mk=%d,", get1bit());
                zprintf(2, "    ES_rate= %d,", getbits(22));
                zprintf(2, "    mk=%d\n", get1bit());
                bytes_used += 3;
              }

              if (DSM_trick_mode_flag == 1)
              {
                trick_mode_control = getbits(3);
                if (verbose_level == 2)
                  printf("  DSM trick mode fields,");
                zprintf(2, "    trick_mode_control= %d\n", trick_mode_control);
                switch (trick_mode_control)
                {
                  case FAST_FORWARD:
                    if (verbose_level == 2)
                      printf("    fast forward fields:");
                    zprintf(2, "      field_id= %d,", getbits(2));
                    zprintf(2, "      intra_slice_refresh= %d,", get1bit());
                    zprintf(2, "      frequency_truncation= %d,", getbits(2));
                    break;

                  case SLOW_MOTION:
                    if (verbose_level == 2)
                      printf("    slow motion fields:");
                    zprintf(2, "      rep_cntrl= %d\n", getbits(5));
                    break;

                  case FREEZE_FRAME:
                    if (verbose_level == 2)
                      printf("    freeze frame fields:");
                    zprintf(2, "      field_id= %d,", getbits(2));
                    zprintf(2, "      reserved= %d\n", getbits(3));
                    break;

                  case FAST_REVERSE:
                    if (verbose_level == 2)
                      printf("    fast reverse fields:");
                    zprintf(2, "      field_id= %d,", getbits(2));
                    zprintf(2, "      intra_slice_refresh= %d,", get1bit());
                    zprintf(2, "      frequency_truncation= %d,", getbits(2));
                    break;

                  case SLOW_REVERSE:
                    if (verbose_level == 2)
                      printf("    slow reverse fields:");
                    zprintf(2, "      rep_cntrl= %d\n", getbits(5));
                    break;

                  default:
                    if (verbose_level == 2)
                      printf("    reserve fields:");
                    zprintf(2, "      reserved= %d\n", getbits(5));
                    break;
                }
                bytes_used++;
              }

              if (additional_copy_info_flag == 1)
              {
                if (verbose_level == 2)
                  printf("  additional copy info:");
                zprintf(2, "    m%d,", get1bit());
                zprintf(2, "    additional_copy_info= %d\n", getbits(7));
                bytes_used++;
              }

              if (PES_CRC_flag == 1)
              {
                if (verbose_level == 2)
                  printf("  PES CRC field:");
                zprintf(2, "    previous_PES_packet_CRC= %d\n", getbits(16));
                bytes_used += 2;
              }

              if (PES_extension_flag == 1)
              {
                if (verbose_level == 2)
                  printf("  PES extension fields:\n");
                PES_private_data_flag = get1bit();
                zprintf(2, "    PES_private_data_f=%d,", PES_private_data_flag);
                pack_header_field_flag = get1bit();
                zprintf(2, "    pack_header_fld_f=%d  ", pack_header_field_flag);
                program_packet_sequence_counter_flag = get1bit();
                zprintf(2, "    program_pkt_seq_ctr_f=%d\n", program_packet_sequence_counter_flag);
                PSTD_buffer_flag = get1bit();
                zprintf(2, "    P-STD_buffer_flag= %d,", PSTD_buffer_flag);
                zprintf(2, "    res= %d", getbits(3));
                PES_extension_flag_2 = get1bit();
                zprintf(2, "    PES_extension_flag_2= %d\n", PES_extension_flag_2);
                bytes_used++;

                if (PES_private_data_flag == 1)
                {
                  if (verbose_level == 2)
                    printf("    PES private data:");
                  for (iPayload = 0; iPayload < 16; iPayload++)
                    getbits(8);
                  zprintf(2, "      read %d bytes of private data\n", iPayload);
                  bytes_used += 16;
                }

                if (pack_header_field_flag == 1)
                {
                  pack_field_length = getbits(8);
                  printf("\n    ****\n    **** PACK HEADER FIELD FLAG=1 INSIDE PES x%02X Stream PKT#%d\n          - NOT ALLOWED in program stream\n    ****\n\n", 
                          stream_id, (stream_packets[stream_id] - 1));          

                  // beep(200,200)
                  //exit(1);
                  for (iPayload = 0; 
                       iPayload < pack_field_length; 
                       iPayload++)
                    getbits(8);
                  zprintf(2, "      read %d bytes of PES PACK HDR\n", iPayload);
                  bytes_used += pack_field_length +1;
                }

                if (program_packet_sequence_counter_flag == 1)
                {
                  if (verbose_level == 2)
                    printf("    program packet sequence counter fields:");
                  zprintf(2, "      mk=%d\n", get1bit());
                  zprintf(2, "      program_packet_sequence_counter= %d,", getbits(7));
                  zprintf(2, "      mk=%d\n", get1bit());
                  zprintf(2, "      MPEG1_MPEG2_identifier= %d,", get1bit());
                  zprintf(2, "      original_stuff_length= %d\n", getbits(6));
                  bytes_used += 2;
                }

                if (PSTD_buffer_flag == 1)
                {
                  if (verbose_level == 2)
                    printf("    P-STD buffer fields\n");
                  zprintf(2, "      01= %d", getbits(2));
                  zprintf(2, "  P-STD_buffer_scale= %d,", get1bit());
                  iTmp1 = getbits(13);
                  zprintf(2, "    P-STD_buffer_size= %d", iTmp1);
                  zprintf(2, "=x%02X\n", iTmp1);
                  bytes_used += 2;
                  if (verbose_level > 3)
                    strcat(level3, " P-STD");
                }

                if (PES_extension_flag_2 == 1)
                {
                  if (verbose_level == 2)
                    printf("    PES extension 2 fields:");
                  zprintf(2, "      mk=%d\n", get1bit());
                  PES_extension_field_length = getbits(7);
                  zprintf(2, "      PES_extension_field_length= %d\n", PES_extension_field_length);
                  bytes_used++;
                  for (iPayload = 0; 
                       iPayload < PES_extension_field_length; 
                       iPayload++)
                  {
                    getbits(8);
                    bytes_used++;
                  }
                  zprintf(2, "      read %d bytes of PES extension field data\n", iPayload);
                }
              }

              PES_HDR_Flush();
}


void PES_HDR_Flush()
{
  int iTmp1, iTmp8;

              iTmp8 = (PES_header_data_length - (bytes_used - 3)); // BYTE-ME

              if (iTmp8 < 0)
              {
                  printf("\n    ****\n    **** PACKET HEADER OVERFLOW %d BYTES in x%02X Stream Pkt#%d\n    ****\n\n",
                        iTmp8, stream_id, (stream_packets[stream_id] - 1));
                  //beep(200,200);
              }
              else
              if (iTmp8)
              {
                zprintf(2, "\n  Read %i Hdr Stuffing bytes: ", iTmp8);
                for (iPayload = 0; iPayload < iTmp8; iPayload++)
                {
                   iTmp1 = getbits(8);
                   zprintf(2, " x%02X", iTmp1);
                }
                if (verbose_level)
                    printf(".\n");
                bytes_used += iTmp8;
              }
}


//------------------------------------------------------------------------

unsigned int uStart_Code_byte;

void video_hdr()
{

  unsigned long uTmp0, uTmp1, uTmp2, uTmp3, uTmp4;
  unsigned long uXTN_type, uProfile, uLvl, uSQX_Progressive, uSQX_Chroma;
  unsigned long uSQX_Horiz_ext, uSQX_Vert_ext;
  unsigned long uVideoFormat, uColorFlag, uDspWidth, uDspHeight;
  unsigned long uTemporal, uTemporal_ms, uPicType, uVBV_Delay;
  // char cTmp1;

  char *szEXTN_TYPE[16]
   = {"Reserved 0", "SEQ EXTN   ", "SQ DISP XTN", "Quant Matrix",
      "Copyright Info", "Sequence Scalable", "Reserved 6", "Pic Display ",
      "Pic Coding ",  "Pic Spatial Scalable", "Pic Temporal Scalable",
      "Reserved B", "Reserved C", "Reserved D", "Reserved E", "Reserved F"};


  char *szAspType[2] = {"Pixel", "Frame"};
  char *szASPECT[2][16]
    = {  // Mpeg1
       {"0  ",      " VGA",      "a.6735",   "a.7031",
        "a.7615",   "a.8055",   "a.8437",   "a.8935",
        "a.9375",   "a.9815",   "a1.0255",  "a1.0695",
        "a1.1250",  "a1.1575",  "a1.2015",  "asp15"},
        // Mpeg2
       {"0  ",  " VGA",   " 4:3",   "16:9",  " 2.21",
        "asp5",  "asp6",  "asp7",
        "asp8",  "asp9",  "asp10", "asp11", "asp12",
        "asp13", "asp14", "asp15"}};


  char szDrop[8];


  char *szPROGRESSIVE[2] = {" Interlaced", "Progressive"};
  char *szCHROMA_FMT[4]  = {"0:0:0", "4:2:0", "4:2:2", "4:4:4"};
  char *szPIC_TYPE[5]    = {"D-0", "I", "P", "B", "UNKNOWN"};

  char *szPICSTRUCT[4] = {"UNK FMT", "TOP First", "BOT First", "Progressive Frame"};
  char *szREPEAT[2] = {" ", "REPEAT"};

  char *szPROFILE[8] = {"0P","HighP", "Spatial", "SNR", "MainP", "SimpleP", "6P","7P"};
  char *szLEVEL[16]  = {"0L",    "1L",     "2L",       "3L",
                        "HighL", "Hi5L",   "High1440", "Hi7L",
                        "MainL", "Main9L", "LowL",     "Low11",
                        "Low12", "Low13",  "Low14",    "Low15"};

  char *szVideoFormat[8] = {"component", "PAL", "NTSC", "SECAM", "MAC", 
                            "5", "6", "7"};

  char szTmp64[64];

  // what type of start code is it ?

  switch (uStart_Code_byte) // look_ahead(8))
  {

      case USER_DATA_START_CODE:

          if (verbose_level == 2 || iFirstPktOfSubStream)
          {
              printf("  * %03d - USER DATA: \"", offset);

              uTmp2 = 42;  // uTmp3 = 42;

              for (uTmp0=0;
                   uTmp0 < 50;
                   uTmp0++)
              {
                 uTmp1    = getbits(8);
                 iPayload++;

                 if (uTmp1)
                 {
                     if (uTmp1 < ' ' || uTmp1 > '}') // Censor unprintable characters
                         printf(".%02X.", uTmp1);
                         //printf(".");
                     else
                         printf("%c", uTmp1);
                 }
                 else
                 {
                   if (uTmp2)
                       printf(".");
                   else
                   {
                       video_sync[uVidTrk] = 2; // Found 2 nulls
                       uTmp0 = 99;  // terminate loop
                    }
                 }

                 // uTmp3 = uTmp2;
                 uTmp2 = uTmp1;

              } // endfor

              printf("\"\n");
          }
          break;



      case SEQUENCE_HEADER_CODE:

          if (offset)
          {
              if (iSeq_ctr)
                  strcpy(szFlag,"UNALIGNED");
              else
                  strcpy(szFlag,"UNALIGNED ***BAD FOR SOME DECODERS\n          ");
              iSeq_Unaligned_ctr++;
          }
          else
              strcpy(szFlag,"Aligned") ;


          if (uRemaining > 4)
          {
              uTmp0       = getbits(32);
              iPayload   +=4;

              uHoriz      = (uTmp0>>20);
              uVert       = (uTmp0>>8) & 0x0FFF;
              uAspectCode = (uTmp0>>4) & 0x0F;
              uFRateCode  = (uTmp0)    & 0x0F;
              uFRate1001  = uFRAME_RATE_1001[uFRateCode];

              if (uHoriz & 7)    // Evenly divisble by 8 ?
                  strcpy(szTmp32, "*BAD-H*\n\t\t\t");
              else
              if (uVert & 7)    // Evenly divisble by 8 ?
                  strcpy(szTmp32, "*BAD-V*\n\t\t\t");
              else
                  strcpy(szTmp32, "");

             if (verbose_level == 2 || iFirstPktOfSubStream
             ||  szTmp32[0] > ' ')
             {

                 printf(
      "  * %03d - SEQ %s  %ux%u %s %s %sAspect  %sFPS(base)\n",
                          offset, szFlag, uHoriz, uVert, szTmp32,
                                                szASPECT[mpeg2][uAspectCode],
                                                szAspType[mpeg2],
                                                szFRAME_RATE[uFRateCode]);
             }
          }

          iSeq_ctr++;
          break;



      case EXTENSION_START_CODE:

          if (verbose_level == 2 || iFirstPktOfSubStream)
          {
             printf("  * %03d - ", offset);
             if (uRemaining < 8)
             {
                 printf("EXTN Hdr Split across packets");
             }
             else
             {
                 uXTN_type = getbits(4);
                 uTmp0     = getbits(20);
                 iPayload +=3;

                 printf("%s ",  szEXTN_TYPE[uXTN_type]) ;

                 if (uXTN_type == 1)   // SEQ EXTN
                 {
                   uTmp1            =  uTmp0 >>12;
                   uProfile = (uTmp1 >>4) & 0x7;
                   uLvl     =  uTmp1      & 0xF;

                   uSQX_Progressive = (uTmp0 >>11)  & 0x01;
                   uSQX_Chroma      = (uTmp0 >> 9)  & 0x03;
                   uSQX_Horiz_ext   = (uTmp0 >> 7)  & 0x03;
                   uSQX_Vert_ext    = (uTmp0 >> 5)  & 0x03;

                   printf(" %dx%d %s display  %s %s@%s  hx%d vx%d ",
                               ((uSQX_Horiz_ext<<12)+uHoriz),
                               ((uSQX_Vert_ext<<12)+uVert),
                               szPROGRESSIVE[uSQX_Progressive],
                               szCHROMA_FMT[uSQX_Chroma],
                               szPROFILE[uProfile], szLEVEL[uLvl],
                               uSQX_Horiz_ext, uSQX_Vert_ext);

                   if (uRemaining > 7)
                   {
                      getbits(7);  // rest of BitRate Extension
                      get1bit();   // Marker bit

                      uFRate_Xtn_n = getbits(2) + 1;
                      uFRate_Xtn_d = getbits(5) + 1;

                      getbits(8);  // VBV Extension
                      get1bit();   // Low Delay Flag
                      iPayload +=3;

                      if (uFRate_Xtn_n != uFRate_Xtn_d)
                      {
                         uFRate1001 = uFRAME_RATE_1001[uFRateCode]
                                    * uFRate_Xtn_n / uFRate_Xtn_d;

                         fFrameRate = float(uFRate1001)
                                    / 1001.0;

                         printf("\n                   Rate*%u/%u = %f FPS",
                                                     uFRate_Xtn_n,
                                                     uFRate_Xtn_d,
                                                     fFrameRate);
                      }
                   }

                 }
                 else
                 if (uXTN_type == 2)   // SEQ DISPLAY EXTN
                 {
                   uVideoFormat =  uTmp0 >>17;
                   uColorFlag   = (uTmp0 >>16) & 0x1;
                   if (uColorFlag)
                   {
                                  getbits(8);
                     uDspWidth  = getbits(14);
                                  get1bit();
                     uDspHeight = getbits(14);
                                  getbits(3);
                     iPayload +=5;
                   }
                   else
                   {
                     uDspWidth  =  (uTmp0 >>2) & 0x3FFF;
                     uDspHeight = ((uTmp0 <<13) & 0x2000) + getbits(13) ;
                                  getbits(3);
                     iPayload +=2;
                   }

                   printf(" %dx%d=HotSpot   VidFmt=%d-%s  ColDesc=%d            ",
                               uDspWidth, uDspHeight,
                               uVideoFormat, szVideoFormat[uVideoFormat],
                               uColorFlag);

                 }
                 else
                 if (uXTN_type == 8) // PIC CODING EXTN
                 {
                    uPicStruct = (uTmp0>>8) & 0x03;
                    uRepeat    = (uTmp0>>1) & 0x01;
                    if (uRepeat)
                    {
                        uRpt_Rel++; uRpt_ctr++;
                    }

                    printf(" %s  %s",
                                      szPICSTRUCT[uPicStruct],
                                         szREPEAT[uRepeat] );
                 }
                 else
                    printf("  Extn type %X ", uXTN_type);

             }
             if (verbose_level == 2 || iFirstPktOfSubStream)
                 printf("\n");

          }
          break;



      case SEQUENCE_END_CODE:

          if (verbose_level == 2 || iFirstPktOfSubStream)
              printf("  * %03d - sequence_end_code\n", offset);
          break;



      case GROUP_START_CODE:

          GOP_Stats_Inc();

          if (uRemaining < 4)
          {
               strcpy(szBuffer, "Hdr Split across packets");
               szTmp64[0] = 0;
          }
          else
          {

              uTmp1 = look_ahead(25); // GOP TimeCode

              uDrop = uTmp1 & 0x01000000;
              if (uDrop)
                  strcpy(szDrop, "DROP ");
              else
                  szDrop[0] = 0;

              uHH   = (uTmp1 & 0x00F80000)>>19;
              uMM   = (uTmp1 & 0x0007E000)>>13;
              uSS   = (uTmp1 & 0x00000FC0)>> 6;
              uFF   = (uTmp1 & 0x0000003F);

              if (uHH < 10)
                  uTmp4 = uHH;
              else
                  uTmp4 = uHH - 10;

              uTmp2 = (((((uTmp4 * 60) + uMM) * 60)
                                            + uSS) * uFRate1001 / 1001)
                                            + uFF;

              sprintf(szBuffer,
         "%s %uh.%02um.%02us.%02uf                                        %uF",
                             szDrop, uHH, uMM, uSS, uFF,  uTmp2);

              if (uGOP_ctr)
              {
                   uTmp3 = uTmp2 - uGOP_Code_Prev;
                   sprintf(szTmp64, "\n         (PREV: %ut  Frames=%uF %u Calc %ums %d Rpts)",
                                          uTmp3, uRel_ctr,
                     ((uGOP_Diff_PTS_ms+20) * uFRate1001 / 1001000),
                       uGOP_Diff_PTS_ms,
                       uRpt_Rel);
              }
              else
              {
                   uGOP_Code_Start = uTmp2;
                   szTmp64[0] = 0;
              }              }
              uGOP_Code_Prev = uTmp2;

              if (verbose_level == 2 || iFirstPktOfSubStream)
                   printf("  * %03d - GOP %s %s\n",
                              offset, szBuffer, szTmp64);

              uGOP_ctr++;
              uRel_ctr = 0;
              break;


      case PICTURE_START_CODE:

           if (verbose_level == 2 || iFirstPktOfSubStream)
           {
             printf("  * %03d - PIC ", offset);
             if (uRemaining < 4)
             {
                 strcpy(szBuffer, "Hdr Split across packets");
                 szTmp64[0] = 0;
             }
             else
             {
               uTmp1 = look_ahead(29); // PIC TemporalRef, Pic_Type

               uTemporal  = uTmp1>>19;
               uPicType   = (uTmp1>>16) & 0x03;
               uVBV_Delay = uTmp1 & 0xFFFF;

               if (uPicType > 4)
                   uTmp0 = 4;
               else
                   uTmp0 = uPicType;

               if (uVBV_Delay == 0xFFFF)
                  strcpy(szBuffer,"Auto");
               else
                  sprintf(szBuffer,"%dms", (uVBV_Delay/90));

               uTemporal_ms = uTemporal * 1001000/uFRate1001;

               if (!uADJ_FLag && stream_id == 0xE0)
               {
                  uADJ_FLag = 1;
                  uStartADJ = uTemporal_ms;
               }

               printf("%s-TYPE   TRef=%02uF =%02ums  VDelay=%s    %02X",
                          szPIC_TYPE[uTmp0], // uPicType,
                          uTemporal, uTemporal_ms,
                          szBuffer, stream_id);

             } // end-else hdr NOT split

             printf("\n");

           } // end-if (verbose_level || iFirstPktOfSubStream)


           uPIC_ctr++;
           uRel_ctr++;
           break;

      case PACK_START_CODE:

           //if (verbose_level)
              printf("  * %02d - PACK HDR INSIDE BROKEN PACKET *************\n", offset);

           break;
  }

  video_sync[uVidTrk] = 0;

}



//----------


void Video_Payload()
{

  int iExpected;
  uVidTrk = stream_id - VIDEO_STREAM_0;

  iPayload = 0;

  if (// iFirstPktOfSubStream ||
     (verbose_level && PTS_DTS_flags))
      printf("\n");

  iExpected = PES_packet_length - bytes_used; // BYTE-ME - 3;

  while (iPayload < iExpected)
  {
     uStart_Code_byte = getbits(8);

     // see if we are part way through a Start Code (00 00 01 xx)

     switch (video_sync[uVidTrk])
     {
       case 0:
              if (uStart_Code_byte == 0x00)
                  video_sync[uVidTrk] = 1; // Found first null
              break;

       case 1:
              if (uStart_Code_byte == 0x00)
                  video_sync[uVidTrk] = 2; // Found 2nd null in a row
              else
                  video_sync[uVidTrk] = 0; // Reset
              break;

       case 2:
              if (uStart_Code_byte == 0x01)
                  video_sync[uVidTrk] = 3; // Found 2nd null in a row
              else
              if (uStart_Code_byte != 0x00)
                  video_sync[uVidTrk] = 0; // Reset
              break;

       case 3:
             // Start Code Prefix found (00 00 01)

             offset = iPayload - 3; // Where did the start Code Start ?
             uRemaining = iExpected - iPayload;

             video_hdr();
     }

     iPayload++;
  }

  if (verbose_level != 2 && iFirstPktOfSubStream)
      printf("\n");

}




//-------------------------------------------


void PS2_Info()
{
  // int iRemain2;
  // int iTmp1, iTmp2, iTmp3, iTmp4;

  iPayload=0;

  iRemain = PES_packet_length - bytes_used; // BYTE-ME - 3;

  if (iRemain > 1)
  {
      if (verbose_level == 2 || substream_packets[substream_id] < 1)
          Hex_Prt(16);

      if (verbose_level)
          printf("\n");

  }// endif remaining

  for (; iPayload < iRemain; iPayload++)
                          getbits(8);
                          
}


//-------------------------------------------

/*
void MPA_Hdr_Bits(unsigned char cMPA_HDR[4])
{

}
*/



void MPA_Audio_Info()
{
  int iTrk, k;
  int iPrevPrevOffset, iAltFrameBytes, iExpected;
  int id;

  iTrk = stream_id - AUDIO_STREAM_0;
  
  iFramesInPkt = 0; iNomFrameSz = 0;
  offset_Prev  = 0; iPrevPrevOffset = 0;

  iExpected = audio_sync[iTrk];

  for (iPayload = 0;
       iPayload < PES_packet_length - bytes_used; // BYTE-ME - 3;
       iPayload++)
  {
       k = getbits(8);
       if (iPayload < 2
       && (verbose_level || iFirstPktOfSubStream)  )
       {
          if (!iPayload)
                printf("    00 = 0x");

          printf("%02X ", k);
       }
       


       if (k == 0xFF)  //  We no longer skip for length of frame
           audio_sync[iTrk] = -2;
       else

       if (audio_sync[iTrk] ==  -2)
       {
          cMPA_Hdr[1] = k;

          uMPA_Layer_ix =    (cMPA_Hdr[1] & 0x06) >> 1;
          uMPA_Layer    = 4 - uMPA_Layer_ix;
                    
                 if (k < 0xE0   // 0xFA && k != 0xF5)  
                 ||  uMPA_Layer_ix == 0)  
                 {
                     audio_sync[iTrk] = -10;  // -1;
                 }
                 else
                 {
                    offset = iPayload - 1;

                    audio_sync[iTrk] = -3;

                    id           =    (k & 0x08) >> 3; // id is opposite of LSF
                    uMPA_25_LSF  = 3-((k>>3)&3); // Mpeg2.5 hijacks low-order bit of syncword

                    if (uMPA_25_LSF > 1) 
                        uMPA_Mpeg25 = 1;
                    else
                        uMPA_Mpeg25 = 0;

                    if (id)   // LowSamplingFrequency Extension uses the old "ID" Flag
                        uMPA_LSF    = 0;
                    else
                        uMPA_LSF    = 1;


                    iFrameBytes     = offset - offset_Prev;
                    iAltFrameBytes  = offset - iPrevPrevOffset;

                    if (verbose_level == 2 
                    || (iFirstPktOfSubStream && iFramesInPkt < 3))
                    {
                        strcpy(szMPA_Note,"            "); // szMPA_Note[0] = 0; 
                        cMPA_Sus = '-';

                        if (iFramesInPkt > 1)
                        {
                           if (iNomFrameSz == iFrameBytes)
                               printf(" OK");
                            else
                           if (iNomFrameSz != iPrevNomFrameSz)
                               printf(" **SUS**\n                                                   Actual Bytes=%d",
                                        iFrameBytes);
                        }

                        if (offset == 0)
                            strcpy(szMPA_Note,"Aligned     ");
                        else
                        if (iFramesInPkt < 2)
                        {
                            if (iFirstPktOfSubStream )
                                strcpy(szMPA_Note,"*UNALIGNED*");
                        }
                        else
                        if (iFrameBytes     != iNomFrameSz
                        &&  iFrameBytes     != iPrevNomFrameSz
                        &&  iAltFrameBytes  != iNomFrameSz
                        &&  iAltFrameBytes  != iPrevNomFrameSz)
                        {
                            strcpy(szMPA_Note, "**SUS**     ");
                            cMPA_Sus = '?';
                        }


                        /*
                        if (verbose_level || iFramesInPkt < 2)
                            sprintf(
                              "\n  %c %04d - x%02X audio syncword %c mp%d%s %s",
                                   cMPA_Sus, offset, k, cMPA_Sus,
                                                (4 - uMPA_Layer_ix),
                                                MPA_EXTN_ABBR[uMPA_25_LSF],
                                                szMPA_Note);
                        */

                    }


                    iPrevPrevOffset = offset_Prev;
                    offset_Prev = offset;
                    uMPA_HdrLoc = offset;
                    iPrevNomFrameSz = iNomFrameSz;

                 }
                            
       }

       else
       if (audio_sync[iTrk] ==  -3)
       {
          cMPA_Hdr[2] = k;

                 BitRate_ix   = (k & 0xF0) >> 4;
                 Freq_ix      = (k & 0x0C) >> 2;
                 padding_bit  = (k & 0x02) >> 1;

                 if (BitRate_ix > 14 || Freq_ix > 2)
                     audio_sync[iTrk] = -24; // REJECT FALSE SYNCWORD
                 else
                 {
                     audio_sync[iTrk] = -4;

                     uMPA_kBitRate = // BitRate_tbl[3 - uMPA_Layer_ix][BitRate_ix];
                                MPA_KBIT_RATE[uMPA_LSF]
                                             [3 - uMPA_Layer_ix][BitRate_ix];

                     iNomFrameSz = ( (int( slots[uMPA_Layer_ix]
                                            * uMPA_kBitRate * 1000
                                            / MPA_SAMPLE_HZ[uMPA_25_LSF][Freq_ix]  //  uFrequency[Freq_ix]
                                           )
                                       + padding_bit
                                       )
                                     * SlotBytes[uMPA_Layer_ix]
                                    );
                 

                    /*
                    if (verbose_level == 2
                    || (iFirstPktOfSubStream && iFramesInPkt < 2))
                    {
                         // iFirstPktOfSubStream = 0;
                         printf("\n           x%02X %d kbps %d kHz",
                                    cMPA_Hdr[2], uMPA_kBitRate, 
                                    (MPA_SAMPLE_HZ[uMPA_25_LSF][Freq_ix] / 1000),
                    }
                    */
                 }
                   
       }
       else
       if (audio_sync[iTrk] ==  -4)
       {
          cMPA_Hdr[3] = k;

          uMPA_Channel_ix   = (k) >> 6;

          // Layer II - restricted combinations of bitrate and mode
          if (uMPA_Layer == 2
          &&  uMPA_Channel_ix == 3 // Mono
          &&  uMPA_kBitRate > 192)
          {
              audio_sync[iTrk] = -11;  // Not a valid MPA header 
          }
          else
          if (uMPA_Layer == 2
          &&  uMPA_Channel_ix != 3 // NOT Mono
          &&  (   uMPA_kBitRate == 32 || uMPA_kBitRate == 48
               || uMPA_kBitRate == 56 || uMPA_kBitRate == 80
              )
             )
          {
              audio_sync[iTrk] = -12;  // Not a valid MPA header 
          }
          else
          {
              iFramesInPkt++;

              if (verbose_level == 2
              || (iFirstPktOfSubStream && iFramesInPkt < 2))
              {
                  if (offset < 3)
                      strcpy(szTmp32, "PRE");
                  else
                      strcpy(szTmp32, "   ");
                         
                  // iFirstPktOfSubStream = 0;

                  printf("\n%s %04d %c mp%d%s %dkbps %dkHz %s %s FF%02X %02X%02X +%d NomBytes=%d",
                          szTmp32, uMPA_HdrLoc, cMPA_Sus, 
                          uMPA_Layer, MPA_EXTN_ABBR[uMPA_25_LSF],
                          uMPA_kBitRate, 
                         (MPA_SAMPLE_HZ[uMPA_25_LSF][Freq_ix] / 1000),
                          MPG_CH_MODE_ABBR[uMPA_Channel_ix],
                          szMPA_Note,
                          cMPA_Hdr[1], cMPA_Hdr[2], cMPA_Hdr[3],
                          padding_bit,
                          iNomFrameSz);
                 }

                 // Do not know about free format 
                 if (BitRate_ix == 0)
                     audio_sync[iTrk] = -10;
                 else
                     audio_sync[iTrk] = -20; // iNomFrameSz - 4;
          }
                                    
       }
       else
       {
             audio_sync[iTrk]--;  //  WAS remaining length of frame, but not any more
       }
    
       iExpected--;

  }  // endfor

  if (verbose_level == 2 || iFirstPktOfSubStream)
     printf("...\n");
}

///------------------
/*                     
  i = stream_id - AUDIO_STREAM_0;
                     
  offset_Prev = 0;

  for (iPayload = 0; 
       iPayload < PES_packet_length - bytes_used; 
      iPayload++)
      {
                      k = getbits(8);
                      if (!iPayload)
                      {
                         if (verbose_level)
                             printf("   00 = 0x%02X\n", k);
                      }

                      switch (audio_sync[i])
                      {
                        case -1:
                          if (k == 0xFF)
                            audio_sync[i] = -2;
                          break;

                        case -2:
                          if ((k & 0xF0) == 0xF0)
                          {
                            offset = iPayload - 1;
                            uMPA_Layer_ix = (k & 0x06) >> 1;
                            audio_sync[i] = -3;
                            if (verbose_level)
                               printf("    %04d - audio syncword, mp%d, PrevFrameLen=%d\n", offset, (4 - uMPA_Layer_ix), (offset-offset_Prev));
                            offset_Prev = offset;
                          }
                          else
                            audio_sync[i] = -1;
                          break;
                        case -3:
                          BitRate_ix = (k & 0xF0) >> 4;
                          Freq_ix = (k & 0x0C) >> 2;
                          padding_bit = (k & 0x02) >> 1;
                          uMPA_kBitRate = BitRate_tbl[3 - uMPA_Layer_ix][BitRate_ix];
                          audio_sync[i] = (( uMPA_kBitRate * 1000 
                                             / uFrequency[Freq_ix]
                                           ) 
                                           * slots[uMPA_Layer_ix]
                                          ) - 4;
                          if (padding_bit)
                            audio_sync[i] += 1;

                          if (verbose_level)
                             printf("           %d kbps, %d kHz,  Pad=%d Span=%d\n", uMPA_kBitRate, uFreq_k[Freq_ix], padding_bit, audio_sync[i]);

                          if (padding_bit)
                            audio_sync[i]++;
                          break;
                        default:
                          audio_sync[i]--;
                      }
                    }



//-----------------------

  
                  i = stream_id - VIDEO_STREAM_0;
                  for (iPayload = 0; iPayload < PES_packet_length - bytes_used; iPayload++)
                  {
                    k = getbits(8);
                    switch (video_sync[i])
                    {
                      case 0:
                        if (k == 0x00)
                          video_sync[i] = 1;
                        break;
                      case 1:
                        if (k == 0x00)
                          video_sync[i] = 2;
                        else
                          video_sync[i] = 0;
                        break;
                      case 2:
                        if (k != 0x01)
                        {
                          if (k != 0x00)
                            video_sync[i] = 0;
                        }
                        else
                        {
                          offset = iPayload - 2;
                          switch (look_ahead(8))
                          {
                            case USER_DATA_START_CODE:
                              printf("  * %02d - user_data_start_code\n", offset);
                              break;
                            case SEQUENCE_HEADER_CODE:
                              if (offset)
                              {
                                 memcpy(&szFlag,"UNALIGN", 8);
                                 iSeq_Unaligned_ctr++;
                              }
                              else
                                 szFlag[0] = 0x00;

                              printf("  * %02d - sequence_header_code     === SEQ %s\n",
                                         offset, szFlag[0]);
                              iSeq_ctr++;
                              break;
                            case EXTENSION_START_CODE:
                              printf("  * %02d - extension_start_code\n", offset);
                              break;
                            case SEQUENCE_END_CODE:
                              printf("  * %02d - sequence_end_code\n", offset);
                              break;
                            case GROUP_START_CODE:
                              GOP_Stats_Inc();
                              //if (verbose_level)
                                 printf("  * %02d - group_of_pictures header *** GOP   PrevSpan=%u frames  %u ms(PTS)\n",
                                                          offset, uRel_ctr, uGOP_Diff_PTS_ms);
                              uGOP_ctr++;
                              uRel_ctr=0; uRpt_Rel = 0;
                              break;
                            case PICTURE_START_CODE:
                              printf("  * %02d - picture_start_code       --- PIC\n", offset);
                              uPIC_ctr++;
                              uRel_ctr++;
                              break;
                          }
                          video_sync[i] = 0;
                        }
                    }
                  }

*/